/*
 *****************************************************************************
 *
 *        Copyright 1989, 1990, Xylogics, Inc.  ALL RIGHTS RESERVED.
 *
 * ALL RIGHTS RESERVED. Licensed Material - Property of Xylogics, Inc.
 * This software is made available solely pursuant to the terms of a
 * software license agreement which governs its use.
 * Unauthorized duplication, distribution or sale is strictly prohibited.
 *
 * Module Description:
 *
 * 	Security Policy - Annex Security Server
 *
 * Original Author: Dave Harris		Created on: July 1986
 *
 * Module Reviewers:
 *
 *	lint harris parker
 *
 * Revision Control Information:
 *
 * $Id: acp_policy.c,v 1.80.102.6 1995/05/24 14:58:09 carlson Exp $
 *
 * This file created by RCS from:
 *
 * $Source: /nfs/annex/release9.2/src/erpcd/RCS/acp_policy.c,v $
 *
 * Revision History:
 *
 * $Log: acp_policy.c,v $
 * Revision 1.80.102.6  1995/05/24  14:58:09  carlson
 * Error codes now start with ACPU_ so as not to conflict with standard
 * Unix error codes.
 *
 * Revision 1.80.102.5  1995/02/09  10:26:42  tsang
 * Increased the log file message buffer to 3 times ACP_MAXLSTRING to prevent
 * overwriting memory when long invalid usernames were entered.
 *
 * Revision 1.80.102.4  1994/12/28  14:55:58  defina
 * Function ppp_slip_logout was missing a terminate_session() call. This caused
 * duplicate ppp or slip logouts intermittently. See SPR 4008.
 *
 * Revision 1.80.102.3  1994/10/13  14:55:53  bullock
 * spr.3637,3644 - Need to be careful to null terminate Phone number string
 * and access code.  Reviewed by Russ L.
 *
 * Revision 1.80.102.2  1994/09/26  15:03:54  dfox
 * IPX dialback only compares the numbers when verifying if a call back
 * number requested matches one in the acp_userinfo file.
 * Added function strnumcmp().
 *
 * Revision 1.80.102.1  1994/09/20  15:19:47  defina
 * Added the DIALBACK_DELAY parameter.
 *
 * Revision 1.80  1994/09/08  18:27:14  sasson
 * SPR 3418: accept users in acp_userinfo with no accesscode entry.
 * SPR 3409: minimize usage of gethostbyname() when searching acp_dialup.
 * Review by Russ Lamoreaux.
 *
 * Revision 1.79  1994/09/02  12:26:06  sasson
 * SPR 3404: changed appletalk_profile() to reject requests if user
 * profile does not have any at_xxx statement.
 *
 * Revision 1.78  1994/08/26  16:35:56  carlson
 * SPR 2940 -- modifications to support Linux Slackware 1.2.
 *
 * Revision 1.77  1994/08/18  10:32:45  carlson
 * Updated to make new NVE and IPX code compatible with 64 bit machines.
 *
 * Revision 1.76  1994/08/17  15:53:28  bullock
 * spr.3091 - Changed ipx_validate so that call to get_user_access checks
 * for match on both username and accesscode being "ipx".  If it doesn't
 * match on both, it's not a dialback.
 *
 * Revision 1.75  1994/08/04  11:00:42  sasson
 * SPR 3211: call to strcmp() has unused argument.
 *
 * Revision 1.74  1994/07/20  12:28:09  carlson
 * SPR 3093 -- repaired broken syntax in ipx_check_dialback.
 *
 * Revision 1.73  1994/07/15  16:17:25  bullock
 * In case user didn't enter a phone number when doing IPX chargeback,
 * just allow login and don't try to call back.
 *
 * Revision 1.72  1994/07/07  10:11:05  bullock
 * Changes for ipx dialback, too many to name, but mostly in the routines
 * ipx_check_dialback, ipx_validate and access_code_validate
 *
 * Revision 1.71  1994/06/14  09:08:46  carlson
 * SPR 2800 -- ppp_security() must call ppp_security_authorize(),
 * not serial_validate_authorize(), which is for ipx.
 *
 * Revision 1.70  1994/05/23  12:49:15  reeve
 * Properly initialize CLI's accesscode's phone number.
 *
 * Revision 1.69  94/05/22  15:35:58  reeve
 * Make sure check for existence of a phone number within an accesscode
 * is for IPX only.
 * 
 * Revision 1.68  94/05/19  14:12:31  reeve
 * Changed check_dialback()'s local var, Access_code from
 * length of ACP_MAXSTRING to ACP_MAXLSTRING.
 * 
 * Revision 1.67  94/05/17  17:07:23  reeve
 * Merge from IPX tree.
 * 
 * Revision 1.66  1994/04/22  17:27:20  carlson
 * Added check for hmask to rcode generation -- makes modifications
 * easier.
 *
 * Revision 1.65  1994/04/19  16:30:17  carlson
 * SPR 2665 -- USER_VALIDATION flag was broken by an extra return call.
 *
 * Revision 1.64  1994/04/19  16:02:01  carlson
 * SPR 2650 -- one of the SecurID modifications accidently turned
 * off password and account aging for shadow files.
 *
 * Revision 1.63  1994/04/18  14:37:40  sasson
 * Support for long strings is missing in acp_securid_validate().
 *
 * Revision 1.62  1994/04/15  17:02:56  sasson
 * SPR 2613: added missing step in acp_securid_validate.
 *
 * Revision 1.61  1994/04/07  10:23:15  defina
 * Made a fix to stop erroneous logins to occur when the annex
 * fails on a port_to_annex_authorize call.
 *
 * Revision 1.60  1994/03/29  14:25:16  carlson
 * SPR 2574 -- Fix for compilation problems with shadow files.
 *
 * Revision 1.59  1994/03/16  15:23:01  carlson
 * Added support for long strings from promptstring and fixed default
 * for CLI mask.
 *
 * Revision 1.58  1994/03/10  15:05:21  carlson
 * SPR 2113 -- Added both support for hook-callback routines and some
 * demonstration code.
 *
 * Revision 1.57  1994/02/23  14:53:31  carlson
 * SPR 3268 -- removed DIALUP_SLIP option.
 *
 * Revision 1.56  1994/02/17  17:10:36  defina
 * *** empty log message ***
 *
 * Revision 1.55  1994/02/10  10:35:24  barnes
 * SPR 2303: make changes for new include files and library routines
 * required for ACE/Server v1.2
 *
 * Revision 1.54  1994/01/21  10:01:37  carlson
 * Added FTP service security check to ppp security (need user/pass).
 *
 * Revision 1.53  1993/12/30  13:05:31  carlson
 * SPR 2084 -- made compatible with 64 bit machines.
 *
 * Revision 1.52  1993/12/15  12:53:06  carlson
 * SPR 2068 -- added a simple means of enabling both SecurID and ACP
 * at the same time.
 *
 * Revision 1.51.1.1  1994/02/14  10:00:24  grant
 * Added support for NVE filters in request_appletalk_profile() rpc.
 *
 * Revision 1.51  1993/11/02  10:52:25  sasson
 * spr 2195: added time delay to allow dialback call on the
 * same port requesting it.
 *
 * Revision 1.50  1993/10/26  12:33:53  sasson
 * spr 2173: changed calls to promptstring() to check for negative
 * return codes (some sort of srpc error like "timeout").
 *
 * Revision 1.49  1993/10/15  17:01:58  bullock
 * spr.1940 - Use the per-user climask set in acp_userinfo if
 * one exists
 *
 * Revision 1.48  1993/10/08  17:54:46  sasson
 * Added some more log messages for dialback security (spr 2108).
 *
 * Revision 1.47  1993/10/06  14:59:42  bullock
 * spr.2125 - Picky ANSI compiler wants parameters to be of the
 * exact type the routine being called specifies - lots of casting
 * does the trick
 *
 * Revision 1.46  1993/08/24  17:26:45  sasson
 * Added define option (DEFAULT_NO_USERINFO) that allows users who do not
 * have entries in acp_userinfo to dial in into the annex.
 *
 * Revision 1.45  93/07/29  10:52:25  grant
 * Missing an arg to return_appletalk_profile during the ENOUSER case.  This
 * caused the Annex to always timeout ACP calls for users not in the
 * userinfo file.  Also allowed for guest access to be optional in the
 * userinfo file.
 * SPR1902 Reviewed by Paul C.
 * 
 * Revision 1.44  93/05/26  18:37:26  reeve
 * Fixed infinite-loop including of shadow.h.
 * 
 * Revision 1.43  1993/05/24  13:32:31  wang
 * Fixed Spr1502, SecurID is working in slave port problem.
 *
 * Revision 1.42  93/05/04  14:46:29  barnes
 * add code for Kerberos 4 authentication option
 * 
 * Revision 1.41  93/03/30  01:11:16  grant
 * Added appletalk user profile call.
 * 
 * Revision 1.40  1993/03/17  18:37:14  carlson
 * Making it worse -- sprintf shouldn't be declared for AIX.
 *
 * Revision 1.39  93/03/05  18:28:41  sasson
 * Added support for dialback security.
 * 
 * Revision 1.38  93/03/03  07:59:13  barnes
 * add support for securid card
 * 
 * Revision 1.37  93/02/09  11:42:54  bullock
 * spr.678 - in wild_match, call inet_match instead of inet_address so that
 * we can check for multiple IP addresses for the same host
 * 
 * Revision 1.36  93/01/26  10:49:51  carlson
 * Updated to prompt for a new password when the old one expires.
 * 
 * Revision 1.35  93/01/21  11:01:54  carlson
 * SPR 1285 -- added support for shadow file.  Allows protection of
 * encrypted passwords, password aging, account aging, and warning of
 * users.  (Account inactivity detection not yet implemented.)
 * 
 * Revision 1.34  92/08/04  18:37:07  mildram
 * Wanted to know WHO is rejected when Port_passwd fails, thus added:
 * strcat ( Message, Name )  followed by strcat ( Message, ",port_pwd_fail" )
 *        so that Message is not null in this case when given to log_message.
 * 
 * Revision 1.33  92/04/16  17:18:03  carlson
 * Don't loop infinitely if you can't find your dialup file.
 * 
 * Revision 1.32  92/03/17  14:41:13  reeve
 * Put in some debugging code and debugging switch for
 * acp_dialup access.
 * 
 * Revision 1.31  92/02/28  14:34:11  barnes
 * add logging for LAT connect
 * 
 * Revision 1.30  92/02/25  20:33:18  reeve
 * dialup_address_validate() now handles expaned acp_dialup format.
 * 
 * Revision 1.28  92/02/06  17:40:26  reeve
 * Use htonl for local and remote addresses after extraction from
 * acp_dialup file.
 * 
 * Revision 1.27  92/01/08  17:43:19  carlson
 * Added support for ppp/slip global-accept and fixed ppp/slip log messages.
 * 
 * Revision 1.26  92/01/07  16:46:22  carlson
 * Added support for log w/o password and PPP/SLIP disconnect logging.
 * 
 * Revision 1.25  91/12/08  16:53:34  reeve
 * In dialup_address_validate, if end of acp dialup file
 * is reached,  break out of loop and close the file.
 * 
 * Revision 1.24  91/11/26  14:43:45  reeve
 * Send back REQ_PENDING immediately (to keep Annex satisfied)
 * on PPP security and dialup address requests.
 * 
 * Revision 1.22  91/09/24  12:51:37  emond
 * Some comments around the lookup_userport() which administrators are suppose
 * to write.
 * 
 * Revision 1.21  91/09/23  13:15:32  raison
 * change int (baud) to u_short to conform to ntohs() call.
 * 
 * Revision 1.20  91/08/30  16:32:34  sasson
 * Added new parameter (tcp_port) to annex_to_net().
 * 
 * Revision 1.19  91/06/05  13:23:00  emond
 * Put in sleep so dial-up slip will work with modem control lines.
 * 
 * Revision 1.18  91/05/17  10:58:47  grant
 * Famous DIALUP_SLIP fix
 * 
 * Revision 1.17  91/04/26  13:00:26  emond
 * Assure all calls to promptstring() passed sockaddr ptr. Problem showed-up
 * as PORT_PASSWORD not working properly.
 * 
 * Revision 1.16  91/04/08  22:01:58  emond
 * Changed to accommodate interface to TLI
 * 
 * Revision 1.15  90/04/27  11:14:46  loverso
 * Fix problem with DIALUP_SLIP code; now it works
 * 
 * Revision 1.14  90/03/20  23:57:54  loverso
 * Preliminary dial-up slip code (#ifdef'd with DIALUP_SLIP)
 * 
 * Revision 1.13  90/03/06  23:07:23  loverso
 * "Port password" code; option per CLI; #ifdef'f on PORT_PASSWORD
 * (see acp_policy.h)
 * 
 * Revision 1.12  89/04/05  12:21:06  loverso
 * Changed copyright notice
 * 
 * Revision 1.11  88/08/05  14:39:05  harris
 * Added lots of comments.  Don't prompt unless USER_VALIDATION is set.
 * Added wildcard extensions to acp_restrict file, Annexes and Hosts.
 * 
 * Revision 1.10  88/06/15  16:43:44  harris
 * Impproved acp_keys parsing - allow various backslash escapes, allow commas
 * and other significant punctuation in the encryption key.  Keep track of
 * malloc'ed memory in a separate list for SIGHUP handling.
 * 
 * Revision 1.9  88/06/10  15:48:08  harris
 * Change acp_secure to acp_keys.
 * 
 * Revision 1.8  88/05/31  17:04:10  parker
 * fix build problems with XENIX/SLIP
 * 
 * Revision 1.7  88/05/24  18:21:12  parker
 * Changes for new install-annex script
 * 
 * Revision 1.6  88/05/19  17:56:20  harris
 * Necessary changes to expand key to 16 byte password plus crypt tables.
 * 
 * Revision 1.5  88/05/13  17:00:24  harris
 * Allow multiple username password entries.
 * 
 * Revision 1.4  88/05/04  17:49:46  harris
 * New functions - secure_cache() reads ACP_SECURE file, and caches Annexs
 * and their passwords.  Annex_key() retrieves an ecnryption key (hashed pwd).
 * 
 * Revision 1.3  88/04/12  19:20:29  parker
 * fixes for Xenix
 * 
 * Revision 1.2  87/08/13  17:48:28  parker
 * fixed cli login to use MAX_RETRIES
 * 
 * Revision 1.1  87/07/28  11:52:56  parker
 * merged in security
 * 
 *
 * This file is currently under revision by:
 *
 * $Locker:  $
 *
 *****************************************************************************
 */


/* Include Files */

#include "../inc/config.h"

#include "port/port.h"
#include <strings.h>
#include <ctype.h>
#include <stdio.h>
#include <errno.h>
#include <pwd.h>
#include <ndbm.h>

#include "port/install_dir.h"
#include "acp.h"
#include "acp_policy.h"
#include "getacpuser.h"
#include <netdb.h>
#include <sys/types.h>
#include <netinet/in.h>

/* includes for check_servers */
#include <sys/socket.h>
#include <signal.h>
#include <setjmp.h>

extern int timeout();
jmp_buf env;

#ifdef USESHADOW
#ifdef NATIVESHADOW
#include "shadow.h"
#else
#include "ashadow.h"
#endif
#endif

extern int acp_timer_enabled;

#ifdef KERBEROS
#include <sys/param.h>
#include <krb.h>
#include <des.h>
 
#define KRB_TK_DIR "/tmp/tkt_erpcd_"
#endif
  
#ifdef SECURID_CARD
#ifdef ACE1_1
#include "../sdclient/sdcli.h"
#endif
#ifdef ACE1_2
#include "../sdclient/sdi_athd.h"
#include "../sdclient/sdi_size.h"
#include "../sdclient/sdi_type.h"
#include "../sdclient/sdacmvls.h"
#include "../sdclient/sdconf.h"
union config_record configure;
#define CANCELLED 1
#endif
#endif
 
#ifndef TLI
#include <sys/socket.h>
#endif
#include "../inc/erpc/netadmp.h"

/*   Function declarations   */

#ifdef USESHADOW
int warn_user();
#endif

int	is_slip();

int	getacpdialup(), setacpdialup(), endacpdialup();
int	acp_validate(), dialup_address_validate();

#ifdef SECURID_CARD
int	acp_securid_validate();
#endif
char	*crypt();

#ifdef PORT_PASSWORD
void	acp_special();
int	acp_getusr();
#endif

#ifndef SYS_V
#ifndef AIX
#ifndef LINUX
char	*sprintf(); 
#endif
#endif
#endif

char *test_password(),*change_password();

char		*inet_number();
UINT32		inet_address(), hash();
int		check_port_pool();
void		check_dialback();
void		ipx_check_dialback();
void		dialback_log();
int		access_code_validate();
int		findacpdialup();

/*   Global data declarations   */

int	debug_dialup = 0;

INT32	daysleft;	/* Used to print password & account warnings */

/* External data */

extern int	debug,errno;
extern char	*sys_errlist[];

/*
 * Un-comment this line to get debugging
 * information about dialup slip and ppp.
 */
/* #define DEBUG_DIALUP 1 */

/* Used by the shadow-password-file code for aging passwords. */
static char old_password[16];

#ifdef ALLOW_HOOKS
/* Used to force a command into a user session for macro-per-user */
static char stuff_command[80];
#endif

char	unknown[] = "<unknown>";

/*
 *   Define the Annex acp encryption key/password list
 *      (wildcard table and normal table)
 */

typedef	struct	key_list
	{
		UINT32		addr;	/* internet address (number) */
		KEYDATA		*ckey;	/* crypt key and tables */

	}	KEY_LST;

KEY_LST		Wild[MAX_WILD], Tame[MAX_TAME];

int		Nwild = 0, Ntame = 0;	/* number of actual entries */

/*
 *  Keep track of allocated memory to free on SIGHUP signal
 */

char		*Allocated[MAX_KEYS];	/* list of addresses to free */
int		Nmalloced = 0;		/* number of allocated KEYDATA's */

#define LINE \
	puts("\n-----------------------------------------------------------------------");


/*
 *	Annex Security Policy Procedures
 *
 *	global_init()		Annex boot, or security system restart
 *
 *	ppp_security()		Grant or deny access to Annex PPP or SLIP port
 *
 *	dialup_address()	Grant or deny access to Annex dialup user. 
 *
 *	port_to_annex()		Grant or deny access to Annex CLI
 *	port_to_annex_logout()	Log message when CLI is terminated
 *
 *	annex_to_net()		Allow connection from Annex to a Host
 *	annex_to_net_logout()	Log message when connection is closed
 *
 *	annex_to_lat()		Log LAT connection attempt
 *
 *	net_to_port()		Grant or deny access to a port via port server
 *	net_to_port_logout()	Log message when connection is broken
 *
 *	appletalk_profile()	Return per user AppleTalk information 
 *
 *	Refer to ./policy.doc for information on customizing a security policy
 *	See Chapter 8 of the Annex Network Administrator's Guide (Security)
 *
 *	FILES:
 *
 *	In the examples below, INSTALL_DIR is selected by the install-annex
 *	script, and supplied to the erpcd makefile
 *
 *	INSTALL_DIR/acp_keys	    Annex encryption keys, see ENCRYPTION
 *	INSTALL_DIR/acp_passwd	    Annex password file, like /etc/passwd
 *	INSTALL_DIR/acp_restrict    Connectivity restrictions for annex_to_net
 *	INSTALL_DIR/acp_dialup	    Annex dialup addresses file
 *	INSTALL_DIR/acp_logfile	    Security Event Logging File
 *	INSTALL_DIR/acp_user	    Per User Configuration File
 *
 *	ENCRYPTION:
 *
 *	Messages between the security server and the Annex are encrypted if
 *	the Annex NA parameter acp_key is set, and the Annex is subsequently
 *	booted, given the NA "reset annex security" command, or CLI superuser
 *	"passwd" command.  Encryption keys for the server are stored in the
 *	acp_keys file.  Annexes with no entries are assumed to have no key
 *	set.  Since wildcards can be used, it might be necessary to include
 *	an entry in this file by explicitly declaring "no key", for example:
 *
 *	annex01, annex02:	seKret2
 *	#  131.21 net Annexes have same key except for 3 Annexes
 *	131.21.*:		Gub-Net
 *	131.21.1.1:		SpeciaL
 *	131.21.2.1, 131.21.2.2:
 *
 *	Keys on the original Annex are limited to 7 characters, on the Annex II
 *	15 characters.  Any alphanumeric character and most special characters
 *	may be included in a key.
 */


/*
 *	global_init()
 *
 *	This interface is called when the Annex boots, and security is enabled
 *	or if security is enabled for the first time by setting enable_security
 *	to Y (with NA) followed by a "reset annex security" or CLI passwd
 *	command.  In the first case, the Message is the boot image; in the
 *	latter case, the Message is "(restart)".
 */

global_init(Acp, logid, inet, Message, to)

ACP		*Acp;		/* Handle to pass back to library calls */
UINT32		logid,		       /* Log sequence number (initial) */
		inet;		              /* Annex Internet address */
char		*Message;	        /* image booted, or "(restart)" */
struct sockaddr_in *to;		/* who sent the message we will respond to */
{
	/*  Acknowledge call with a generic return message  */
	(void)acp_acknowledge(Acp, to);

	/*  Log a message with initial sequence number and image name  */

	log_message(inet, logid, 0, SERVICE_SECURITY, EVENT_BOOT, Message);

	/*  Exit this session  */

	terminate_session();

	/*  Return - unlikely since terminate_session() exit()s  */

	return;		/*  void  */
}

/*
 * 	dialup_address	
 *
 *	This interface is called whenever a user must secure a port for
 *	dialup access. 
 *	Virtual CLI's and Port-Multiplexed ports use Virtual port numbers.
 *
 *	For Annex	Virtual CLI's		Host Multiplexed
 *	with N ports	start at port		windows for port 1
 *
 *						a   b   c   d
 *
 *	16		65			1, 17, 33, 49
 *	32		129			1, 33, 65, 97
 */

dialup_address(Acp, logid, inet, port, service, Name, loc, rem, to)

ACP		*Acp;		/* Handle to pass to library functions */
UINT32		logid,				/* Log sequence number */
		inet;			     /* Annex Internet address */
int		port,		       /* physical/virtual port number */
		service;	   	      /* Expect SERVICE_DIALUP */
ACP_STRING	Name;					   /* Username */
UINT32		loc,rem;		 /* local and remote addresses */
struct sockaddr_in *to;	    /* who sent the message we will respond to */
{
	/* Send the local and remote addresses as a hint.  We may or
	   may not get the same addresses back. */

	(void)return_dialup_address(Acp, REQ_PENDING, loc, rem, to);

	if(dialup_address_validate(Name, inet, &loc, &rem, port)) {
	  (void)dialup_address_authorize(Acp, REQ_GRANTED, loc, rem, to);
	  log_message(inet, logid, port, service, EVENT_PROVIDE, Name);
	}
	else {
	  (void)dialup_address_authorize(Acp, REQ_DENIED, loc, rem, to);
	  log_message(inet, logid, port, service, EVENT_NOPROVIDE, Name);
	}
	return;
}

/*
 *	ppp_security
 *
 *	This interface is called whenever an Annex user logs in on a
 *	PPP port, and the the NA parameters enable_security and 
 *	cli_security are Y and the security type for that port is PAP.
 *	On virtual CLI's, cli_security (a per-port parameter) is 
 *	assumed to be Y.
 *
 *	Virtual CLI's and Port-Multiplexed ports use Virtual port numbers.
 *
 *	For Annex	Virtual CLI's		Host Multiplexed
 *	with N ports	start at port		windows for port 1
 *
 *						a   b   c   d
 *
 *	16		65			1, 17, 33, 49
 *	32		129			1, 33, 65, 97
 */
ppp_security(Acp, logid, inet, port, service, direction, Name, Pass, to)

ACP		*Acp;		/* Handle to pass to library functions */
UINT32		logid,				/* Log sequence number */
		inet;			     /* Annex Internet address */
int		port,		       /* physical/virtual port number */
		service;		    /* Expect SERVICE_{PPP,SLIP} */
short		direction;		       /* direction of service */
ACP_STRING	Name,Pass;		      /* Username and password */
struct sockaddr_in *to;	    /* who sent the message we will respond to */
{
	int valid = 0;

	/*  supply return to procedure call so Annex does not timeout  */

	(void)return_serial_validate(Acp, REQ_PENDING, to);

	if (service == SERVICE_FTP) {
/* Verify username/password for connection to Annex FTP daemon. */
		valid = acp_validate(Name, Pass);
	} else {

/* If you return REQ_GRANTED here, then all connections are allowed */
/* You can use this feature to bypass security for a group of ports */
		if (Pass[0]=='\0' && Name[0]=='\0') {
			(void)ppp_security_authorize(Acp,REQ_DENIED,to);
			return;
		}

/* Password will be blank if we're just event-logging */
		if (Pass[0] == '\0')
			valid = 1;
		else {
			valid = acp_validate(Name, Pass);
#ifdef USESHADOW
			warn_user(Acp,&valid,Name,to);
#endif
		}
	}

	if (valid > 0) {
	  (void)ppp_security_authorize(Acp, REQ_GRANTED, to);
	  log_message(inet, logid, port, service, EVENT_LOGIN, Name);
	} else {
	  (void)ppp_security_authorize(Acp, REQ_DENIED, to);
	  log_message(inet, logid, port, service, EVENT_REJECT, Name);
	}
}



/*
 *	ipx_validate
 *
 *	This interface is called whenever an Annex user logs in on a
 *	IPX port. 
 */
ipx_validate(Acp, logid, inet, port, service, 
			Name, Pass, Phone, Netnum, to)

ACP		*Acp;		/* Handle to pass to library functions */
UINT32		logid,				/* Log sequence number */
		inet;			     /* Annex Internet address */
int		port,		       /* physical/virtual port number */
		service;		    /* Expect SERVICE_IPX */
ACP_STRING	Name,
		Pass,
		Phone;
int		Netnum;
struct sockaddr_in *to;	    /* who sent the message we will respond to */
{
	int valid = 0, passed;
int             error;

#ifdef SECURID_CARD
        /* securid card validation */
            
	passed = (acp_securid_validate(Name, Pass, Acp, to) > 0);
#else
        /* passwd file validation */
	passed = (acp_validate(Name, Pass) > 0);

#endif /*SECURID_CARD*/

	if (passed > 0) {
#if (DEFAULT_NO_USERINFO > 0)
	    Access	ac_info;
	    error = get_user_access(Name, "ipx", &ac_info); 

#endif
	    if ( (service == SERVICE_IPX_DIALBACK) ||
#if (DEFAULT_NO_USERINFO > 0)
		(error == ACPU_ENOUSER || error == ACPU_ENOACC) ||	
#endif
		(check_port_pool(inet,port,(caddr_t)0) != DIAL_SUCC))
		{
                (void)return_serial_validate(Acp, REQ_GRANTED, to);
		log_message(inet, logid, port, service, EVENT_LOGIN, Name);
		}
	    else
		{
		/* this is a dialback request */

		ipx_check_dialback(Acp, logid, inet, port, service, to, 
				Name, Netnum, Phone);
		}
	}
	else {
	    (void)return_serial_validate(Acp, REQ_DENIED, to);
	    log_message(inet, logid, port, service, EVENT_REJECT, Name);
	}


	/*  terminate (exit()) session  */
	terminate_session();
}



/*
 *	ppp_slip_logout()
 *
 *	This function is called remotely by the Annex when the PPP or SLIP
 *	session terminates, but only if enable_security is set to Y when PPP/SLIP begins.
 */

ppp_slip_logout(Acp, logid, inet, port, service, Username, to)

ACP		*Acp;		/* Handle for various ACP calls */
UINT32		logid,		/* Log file sequence number */
		inet;		/* Annex Internet address */
int		port,		/* physical/virtual port number */
		service;	/* service, expect SERVICE_PPP/SLIP */
char		*Username;	/* user associated with port */
struct sockaddr_in *to;		/* who sent the message we will respond to */
{
	/*  generic acknowledge - return to remote caller  */

	(void)acp_acknowledge(Acp, to);

	log_message(inet, logid, port, service, EVENT_LOGOUT, Username);

        /*  terminate (exit()) this session  */
        terminate_session();

        /*  dummy return  */
        return;

}

/*
 *	port_to_annex()
 *
 *	This interface is called whenever a user starts an Annex CLI, and
 *	the NA parameters enable_security and cli_security are both Y.  On
 *	virtual CLI's, cli_security (a per-port parameter) is assumed to be Y.
 *
 *	Virtual CLI's and Port-Multiplexed ports use Virtual port numbers.
 *
 *	For Annex	Virtual CLI's		Host Multiplexed
 *	with N ports	start at port		windows for port 1
 *
 *						a   b   c   d
 *
 *	16		65			1, 17, 33, 49
 *	32		129			1, 33, 65, 97
 */

port_to_annex(Acp, logid, inet, port, service, to)

ACP		*Acp;		/* Handle to pass to library functions */
UINT32		logid,				/* Log sequence number */
		inet;			     /* Annex Internet address */
int		port,		       /* physical/virtual port number */
		service;			 /* Expect SERVICE_CLI{,_HOOK} */
struct sockaddr_in *to;		/* who sent the message we will respond to */
{
	ACP_LSTRING	Name, Pass;
	char		Message[ACP_MAXLSTRING * 3 + 1];
	int		tries, got_str, passed = 0;
	char		nullstr[1];
	Uprof		up;
	int		error;
	char		*userprompt = ACP_USERPROMPT,
			*passprompt = ACP_PASSPROMPT,
			*incorrect = ACP_INCORRECT,
			*permgrant = ACP_PERMGRANTD,
			*permdenied = ACP_PERMDENIED;
	UINT32		hmask = 0,mask = CLI_MASK,rcode = REQ_DENIED;

UINT32	inet_addr, *loc, *rem;
int tport;
int isdialup, nlogin;


#ifdef ALLOW_HOOKS
	/* By default, we're just going to hook bad CLI commands. */
	hmask = CHOOK_BADCMND;
	stuff_command[0] = '\0';
#endif

#ifdef SECURID_CARD
	int use_securid = 0;
	/* Do securid-card validation on requested ports. */
	if (USE_SECURID_CHECK) {
	    use_securid = 1;
	    userprompt = SID_USERPROMPT,
	    passprompt = SID_PASSPROMPT,
	    incorrect = SID_INCORRECT;
	    permgrant = SID_PERMGRANTD,
	    permdenied = SID_PERMDENIED;
	}
#endif

	if(debug)
		puts("policy: port_to_annex");

	nullstr[0] = '\0';

#if	(USER_VALIDATION)

	/*  supply return to procedure call so Annex does not timeout  */

	(void)return_port_to_annex(Acp,REQ_PENDING,mask,nullstr,to);

	Message[0] = '\0';

	/*  prompt for Name and Password until valid or too many tries  */

	got_str = outputstring(Acp,"\n\
Welcome to WestNet - Westchester County's Internet Service\n\n\
To Register as a new user, type 'new' (no quotes) at the login prompt.\n\
If you have an account, enter your username in all lower case.\n\n\
If you have any problems, please call (914) 967-7816 (voice).\n\n", to);


	for(tries = 0; tries < RETRIES_MAX; tries++) {

	    got_str = promptstring(Acp, Name, userprompt, 1,
					 INPUT_TIMEOUT, to);
	    if(got_str < 0)
		continue;
	    if(got_str) {
				
		if (strcmp(Name,"new") == 0) {
    		strcpy(stuff_command,"rlogin westnet.com");
    		passed = 1;		/* passed security test */
			hmask = CHOOK_PROMPTING;	/* Hook CLI prompts */
			goto return_accepted;
		}


#ifdef ALLOW_HOOKS_DEMO
/*
 * This creates a fake user name which doesn't prompt for a password,
 * but which immediately starts a telnet session to the host "library".
 * You could search a file here and start various services based on the
 * user name.  A useful one might be "guest" for guest accounts.
 */
		if (strcmp(Name,"library") == 0) {
		    strcpy(stuff_command,"telnet -s library");
		    passed = 1;		/* passed security test */
		    hmask = CHOOK_PROMPTING;	/* Hook CLI prompts */
		    goto return_accepted;
		}
/*
 * This section creates a dummy user name which when entered prints back
 * some text (a help string in this case).
 *
 * You could extend this to permit no-login features like "fingering" a user
 * from the "Annex username" prompt or checking for mail.
 */
		if (strcmp(Name,"help") == 0) {
		    tries = 0;
		    got_str = outputstring(Acp,"\n\
Enter your user name and password, and you will be given a normal Annex VCLI\n\
which has an added command -- \"rls\" -- which remotely calls \"ls\" on the\n\
erpcd host and passes in any command-line arguments.  This demonstrates\n\
the modifications of the CLI interface possible with this feature.\n\
\nEnter \"library\" at the user-name prompt and you will be connected\n\
directly to the library via telnet.  This demonstrates the macro-per-user\n\
capability also possible with this feature.\n\n",to);
		    if (got_str != 0)
			goto return_accepted;
		    continue;
		}
#endif /* ALLOW_HOOKS_DEMO */

		got_str = promptstring(Acp, Pass, passprompt,
				       0, INPUT_TIMEOUT, to);
		if(got_str < 0)
		    continue;
		}
	    if(got_str == 0)
		break;

#ifdef PORT_PASSWORD
	    if(!got_str)
		break;
#endif

#ifdef SECURID_CARD
	    if (use_securid)
	    /* securid card validation */
		passed = acp_securid_validate(Name, Pass, Acp, to);
	    else
	    /* Do regular ACP security on the other ports. */
		passed = acp_validate(Name, Pass);
	    if (passed > 0)
		break;
#else
	/* passwd file validation */
	    if ((passed=acp_validate(Name, Pass)) > 0)
		break;
#endif /*SECURID_CARD*/

	    (void)outputstring(Acp, incorrect, to);	/* error message */

	    if(strlen(Message))
		    (void)strcat(Message, ",");

	    (void)strcat(Message, Name);		/* log failed user */
	    }

		/* Do not allow more than one log-in per user */
		nlogin = check_server(Name, Message);
		if (nlogin >= 1) {
			got_str = outputstring(Acp, "\n\
  I'm sorry, but someone is already logged in using this account.\n\
  WestNet accounts are for a single dial-in at a time only. if you\n\
  feel that no one else should be using your account, please call us\n\
  at (914) 967-7816 during normal business hours.\n", to);
			passed = 0;
			got_str = 1;
			strcpy (Message, Name);
			strcat (Message, " - already logged in");
			goto return_accepted;
		}
		

		/* SLIP/PPP users get a menu, other rlogin directly now. */
		isdialup = is_slip(Name);
		if (isdialup == 0) {
/*			got_str = outputstring(Acp,
				"\n *** Validating through acp_dialup ***\n",
				to);
*/
			isdialup=dialup_address_validate(Name, inet_addr, 
				&loc, &rem, tport);
		}
/*		else	got_str = outputstring(Acp,
				"\n *** Verified from userdb ***\n",to);
*/

		if (isdialup == 0) {
	    	strcpy(stuff_command,"rlogin westnet.com");
			hmask = CHOOK_PROMPTING;	/* Hook CLI prompts */
	    } else {
			got_str = outputstring(Acp,"\n\
  To connect to WestNet, enter 'w' or 'westnet' (no quotes).\n\
  To start SLIP or PPP, enter 'slip' or 'ppp'.\n\
  If you need help, enter 'help' or '?'.\n\
  When done. enter 'h' or 'hangup'.\n\n",to);
	    } 


#ifdef PORT_PASSWORD
	/*
	 *	Prompt for optional port password
	 */

	if (passed > 0 && got_str) {
		char	portstring[32];

		acp_special(portstring, inet, port);
		if (acp_getusr(portstring)) {

			for (tries = 0; tries < RETRIES_MAX; tries++) {
				got_str = promptstring(Acp, Pass,
							ACP_PORTPROMPT, 0,
							INPUT_TIMEOUT, to);
				if (got_str < 0)
					continue;
				if (!got_str) {
					passed = 0;
					break;
				} else
					passed = acp_validate(portstring, Pass);

				if (passed != 0)
					break;
			}
			if  ( ! passed ) 
				(void) strcat ( Message, Name ) ;
				(void) strcat ( Message, ",port_pwd_fail" ) ;
		}
	}
#endif

#ifdef SECURID_CARD
	if (!use_securid)
#endif
#ifdef USESHADOW
	    warn_user(Acp,&passed,Name,to)
#endif
	    ;

return_accepted:
	if (passed > 0)				/* authorize use of CLI */
	    {
#if (DEFAULT_NO_USERINFO > 0)
	    Access	ac_info;
	    int		ret_user_access;
#endif
	    if ( (service == SERVICE_DIALBACK) ||
#if (DEFAULT_NO_USERINFO > 0)
		 ((ret_user_access = get_user_access(Name, 0, &ac_info))
				== ACPU_ENOUSER) ||
		 (ret_user_access == ACPU_ENOACC) ||
#endif
	         (check_port_pool(inet, port, (caddr_t)0) != DIAL_SUCC))
		{
		/* requesting port is not member of any pool */
		/* this is just a direct connect request     */
		(void)outputstring(Acp, permgrant, to);
		error = get_user_profile(Name,&up);
		if (error == ACPU_ESUCCESS && up.up_climask != 0)
			mask = up.up_climask;
		rcode = REQ_GRANTED;
#ifdef ALLOW_HOOKS
		if (service == SERVICE_CLI_HOOK && hmask != 0)
			rcode = REQ_GRANT_HOOK;
#endif
		error = port_to_annex_authorize(Acp,rcode,mask,Name,
			hmask);

                /* If the Annex does not respond to the autherize call
                   above then something has gone wrong on the Annex end
                   and we do not want to log the login */
                if( error == ACPU_ESUCCESS )
			log_message(inet, logid, port, service, EVENT_LOGIN, Name);
		}
	    else
		{
		/* this is a dialback request */
		check_dialback(Acp, logid, inet, port, service, to, Name);
		}
	    }
	else if(got_str)			/* else reject and log */
	    {
	    (void)outputstring(Acp, permdenied, to);
	    (void)port_to_annex_authorize(Acp,rcode,mask,"",0);
	    log_message(inet, logid, port, service, EVENT_REJECT, Message);
	    }
	else					/* reject and log timeout */
	    {
	    (void)outputstring(Acp, ACP_TIMEDOUT, to);
	    (void)port_to_annex_authorize(Acp,rcode,mask,"",0);
	    log_message(inet, logid, port, service, EVENT_TIMEOUT, Message);
	    }

#else /* !USER_VALIDATION */

	/*  with user validation disabled, return CLI mask, and log */

	rcode = REQ_GRANTED;
#ifdef dont_do_this
#ifdef ALLOW_HOOKS
	if (service == SERVICE_CLI_HOOK && hmask != 0)
	    rcode = REQ_GRANT_HOOK;
#endif
#endif
	(void)return_port_to_annex(Acp,rcode,mask,nullstr,to);
	log_message(inet, logid, port, service, EVENT_LOGIN, unknown);

#endif /* USER_VALIDATION */

	/*  terminate (exit()) session  */

#ifdef ALLOW_HOOKS
	if (service == SERVICE_CLI_HOOK && rcode == REQ_GRANT_HOOK) {
		acp_timer_enabled = 0;
		return;
	}
#endif
	terminate_session();
}


/*
 *	annex_to_net()
 *
 *	This interface is called whenever a CLI user attempts to connect to
 *	a machine with a call, rlogin, or telnet command; OR when a port
 *	configured for dedicated mode attempts to connect to its dedicated
 *	host.  Enable_security and connect_security must be set to Y.
 *
 *	The default policy is to scan the acp_restrict file for entries
 *	which explicitly allow or disallow connection from the given
 *	Annex to the given remote host.  The file is searched sequentially.
 *	If no entry is found which matches both the Annex and remote host,
 *	either by name, Internet address, or wildcard, then permission is
 *	granted.  For example:
 *
 *	*:		snowwhite
 *	annex01:	harp, knight
 *	annex02:	harp, knight
 *	130.12.*~	131.12.*, annex01, annex02
 *	130.12.*:	*
 *
 *	: means restricted host, ~ means unrestricted host.  No Annex may
 *	connect to snowwhite; annex01 and annex02 may not connect to harp or
 *	knight; Annexes on network 130.12 may only connect to other Annexes
 *	on the same network, or to annex01 or annex02.
 */


annex_to_net(Acp, logid, linet, port, service, rinet, Username, to,tcp_port_req)

ACP		*Acp;			/* Handle to pass to library calls */
UINT32		logid,			       /* log file sequence number */
		linet,			  /* Internet address of the Annex */
		rinet;			/* Internet address of target host */
int		port,			/* physical or virtual port number */
		service,		   /* SERVICE_{CALL,RLOGIN, TELNET} */
		tcp_port_req;		/* tcp port number */
char		*Username;		   /* Username associated with CLI */
struct sockaddr_in *to;		/* who sent the message we will respond to */
{
	char	String[64+10];
	char	Hostname[32];

	if(debug)
		puts("policy: annex_to_net");

	/*  Inform Annex that policy decision is pending  */

	(void)return_annex_to_net(Acp, REQ_PENDING, to);

	/*  Translate Remote Internet address for log message  */

	inet_name(Hostname, rinet);

	/*  If USER_VALIDATION, include username in log message  */

#if	(USER_VALIDATION)

	if (tcp_port_req < 0)
	    (void)sprintf(String, "%s:%s", Hostname, Username);
	else
	    (void)sprintf(String, "%s:%d:%s", Hostname, tcp_port_req, Username);

#else

	if (tcp_port_req < 0)
	    (void)strcpy(String, Hostname);
	else
	    (void)sprintf(String, "%s:%d", Hostname, tcp_port_req);

#endif

	/*  Check whether or not this is a restricted host for this Annex  */

	if(available(linet, rinet))		/* if OK, authorize and log */
	{
	    (void)annex_to_net_authorize(Acp, REQ_GRANTED, to);
	    log_message(linet, logid, port, service, EVENT_LOGIN, String);
	}
	else					/* else reject and log */
	{
	    (void)annex_to_net_authorize(Acp, REQ_DENIED, to);
	    log_message(linet, logid, port, service, EVENT_REJECT, String);
	}

	/*  terminate (exit()) this session  */

	terminate_session();

	/*  dummy return which might never happen  */

	return;
}


/*
 *	annex_to_lat()
 *
 *	This interface is called whenever a CLI user attempts to 
 *	connect to a machine with a LAT connect command.
 *	Enable_security and connect_security must be set to Y.
 *
 *	The connection attempts are only logged.
 */


annex_to_lat(Acp, logid, linet, port, service, Username, Service_name, to)

ACP		*Acp;		/* Handle to pass to library calls */
UINT32		logid,		/* log file sequence number */
		linet;		/* Internet address of the Annex */
int		port,		/* physical or virtual port number */
		service;	/* SERVICE_CONNECT */
char		*Username,	/* Username associated with CLI */
		*Service_name;	/* LAT service name */
struct sockaddr_in *to;	  /* who sent the message we will respond to */
{
    char	String[64+10];

    if(debug)
	puts("policy: annex_to_net");

    /*  Inform Annex that policy decision is pending  */

    (void)return_annex_to_net(Acp, REQ_PENDING, to); 
    (void)annex_to_net_authorize(Acp, REQ_GRANTED, to);

    (void)sprintf(String, "%s:%s", Service_name, Username);

    log_message(linet, logid, port, service, EVENT_LOGIN, String);

    /*  terminate (exit()) this session  */

    terminate_session();

    /*  dummy return which might never happen  */

    return;
}


/*
 *	net_to_port()
 *
 *	Interface used by conversational port server when enable_security
 *	and port_server_security are set to Y.  In this case, the remote
 *	host is the host from which the Annex was connected via telnet.
 *
 *	Similar to the CLI security policy.  If user's name/password is
 *	not correct, the Annex will redisplay any rotary names, and will
 *	continue with the port/rotary selection prompt.
 */

net_to_port(Acp, logid, linet, port, service, rinet, to)

ACP		*Acp;			/* Handle to pass to library calls */
UINT32		logid,			       /* log file sequence number */
		linet,			  /* Internet address of the Annex */
		rinet;			/* Internet address of remote host */
int		port,			/* physical or virtual port number */
		service;		/* service expected: SERVICE_PORTS */
struct sockaddr_in *to;		/* who sent the message we will respond to */
{
	char		String[64];
	char		Hostname[32];
	ACP_LSTRING	Name, Pass;
	int		valid;
	char		*userprompt = ACP_USERPROMPT,
			*passprompt = ACP_PASSPROMPT,
			*permgrant = ACP_PERMGRANTD,
			*permdenied = ACP_PERMDENIED;

#ifdef PORT_PASSWORD
	int		got_str;
	char		portstring[32];
#endif

#ifdef SECURID_CARD
	int use_securid = 0;
	/* Do SecurID-card validation on requested ports. */
	if (USE_SECURID_CHECK) {
	    use_securid = 1;
	    userprompt = SID_USERPROMPT,
	    passprompt = SID_PASSPROMPT,
	    permgrant = SID_PERMGRANTD,
	    permdenied = SID_PERMDENIED;
	}
#endif

	if(debug)
		puts("policy: net_to_port");

	/*  Notify Annex of pending security decision  */

	(void)return_net_to_port(Acp, REQ_PENDING, to);

	/*  Translate remote host's address for logging purposes  */

	inet_name(Hostname, rinet);

#if	(USER_VALIDATION)

#ifndef PORT_PASSWORD

	/*  prompt for user's name and password  */

	(void)promptstring(Acp, Name, userprompt, 1, INPUT_TIMEOUT, to);
	(void)promptstring(Acp, Pass, passprompt, 0, INPUT_TIMEOUT, to);

	/*  construct log file string hostname:username  */

	(void)sprintf(String, "%s:%s", Hostname, Name);

	/*  look up name/password and decide if it's valid  */

#ifdef SECURID_CARD			/* securid card validation */
	if (use_securid)
	    valid = acp_securid_validate(Name, Pass, Acp, to);
	else {
#endif /* SECURID_CARD */
	    valid = acp_validate(Name, Pass);
#ifdef USESHADOW
	    (void)warn_user(Acp,&valid,Name,to);
#endif
#ifdef SECURID_CARD
	}
#endif
	if (valid > 0)
	{
	/* authorize and log if valid */
		(void)outputstring(Acp, permgrant, to);
		(void)net_to_port_authorize(Acp, REQ_GRANTED, Name, to);
		log_message(linet, logid, port, service, EVENT_LOGIN, String);
	}
	else				/* else reject and log */
	{
		(void)outputstring(Acp, permdenied, to);
		(void)net_to_port_authorize(Acp, REQ_DENIED, "", to);
		log_message(linet, logid, port, service, EVENT_REJECT, String);
	}

#else /* PORT_PASSWORD */

	/*
	 *	Prompt for name and port password
	 */

	got_str = promptstring(Acp, Name, userprompt, 1, INPUT_TIMEOUT,to);

	/*  construct log file string hostname:username  */
	(void)sprintf(String, "%s:%s", Hostname, Name);

	if(got_str <= 0)
		goto timedout;

#ifndef ONLY_USER
	got_str = promptstring(Acp, Pass, passprompt, 0, INPUT_TIMEOUT,to);
	if(got_str <= 0)
		goto timedout;
#endif

	/* look up name (and maybe password) to decide if it's valid  */
#ifdef ONLY_USER
	valid = acp_getusr(Name);
#else
	valid = acp_validate(Name, Pass);
#endif
	if (valid > 0) {

		acp_special(portstring, linet, port);

		if (acp_getusr(portstring)) {
			got_str = promptstring(Acp, Pass, ACP_PORTPROMPT,
				      0, INPUT_TIMEOUT, to);

			if (got_str <= 0)
			    goto timedout;

			valid = acp_validate(portstring, Pass);
		}
	}

#ifdef USESHADOW
	warn_user(Acp,&valid,Name,to);
#endif
	if (valid > 0) {
		(void)outputstring(Acp, permgrant, to);
		(void)net_to_port_authorize(Acp, REQ_GRANTED, Name, to);
		log_message(linet, logid, port, service, EVENT_LOGIN, String);
	} else {
		(void)outputstring(Acp, permdenied, to);
		(void)net_to_port_authorize(Acp, REQ_DENIED, "", to);
		log_message(linet, logid, port, service, EVENT_REJECT, String);
	}
	goto done;

timedout:
	(void)outputstring(Acp, ACP_TIMEDOUT, to);
	(void)net_to_port_authorize(Acp, REQ_DENIED, "", to);
	log_message(linet, logid, port, service, EVENT_TIMEOUT, String);

done:

#endif /* PORT_PASSWORD */

#else

	/*
	 *  If no user name/password validation is desired,
	 *  authorize unconditionally and log a message
	 */

	(void)net_to_port_authorize(Acp, REQ_GRANTED, "", to);
	(void)sprintf(String, "%s:%s", Hostname, unknown);
	log_message(linet, logid, port, service, EVENT_LOGIN, String);

#endif

	terminate_session();
	return;
}


/*
 *	port_to_annex_logout()
 *
 *	This function is called remotely by the Annex when the CLI terminates.
 *	Only if cli_security and enable_security were set to Y when CLI began.
 */

port_to_annex_logout(Acp, logid, inet, port, service, Username, to)

ACP		*Acp;			/* Handle for various ACP calls */
UINT32		logid,			    /* Log file sequence number */
		inet;			      /* Annex Internet address */
int		port,			/* physical/virtual port number */
		service;		 /* service, expect SERVICE_CLI */
char		*Username;		   /* user associated with port */
struct sockaddr_in *to;		/* who sent the message we will respond to */
{
	/*  generic acknowledge - return to remote caller  */

	(void)acp_acknowledge(Acp, to);

	/*  log a message for accounting/security purposes  */

#if	(USER_VALIDATION)

	log_message(inet, logid, port, service, EVENT_LOGOUT, Username);

#else

	log_message(inet, logid, port, service, EVENT_LOGOUT, unknown);

#endif

	/*  terminate this session, which exit()s  */

	terminate_session();

	/*  dummy return  */

	return;
}


/*
 *	annex_to_net_logout()
 *
 *	This function is called when a session of a CLI or dedicated port
 *	ends for whatever reason. (reset, inactivity, hangup, kill, logout)
 */

annex_to_net_logout(Acp, logid, linet, port, service, rinet, Username, to)

ACP		*Acp;			/* Handle for ACP communication */
UINT32		logid,			 /* sequence number for logging */
		linet,			      /* Annex Internet address */
		rinet;			/* Internet addr of remote host */
int		port,				   /* Annex port number */
		service;		/* SERVICE_{RLOGIN,CALL,TELNET} */
char		*Username;		   /* user associated with port */
struct sockaddr_in *to;		/* who sent the message we will respond to */
{
	char	String[64];
	char	Hostname[32];

	/*  generic acknowledge - return to remote caller  */

	(void)acp_acknowledge(Acp, to);

	/*  log a message for accounting/security purposes  */

	inet_name(Hostname, rinet);

	/*  If USER_VALIDATION, include username in log message  */

#if	(USER_VALIDATION)

	(void)sprintf(String, "%s:%s", Hostname, Username);

#else

	(void)strcpy(String, Hostname);

#endif
	log_message(linet, logid, port, service, EVENT_LOGOUT, String);

	/*  terminate this session, which exit()s  */

	terminate_session();

	/*  dummy return  */

	return;
}


/*
 *	annex_to_lat_logout()
 *
 *	This function is called when a session of a LAT connect
 *	ends for whatever reason. 
 */

annex_to_lat_logout(Acp, logid, linet, port, service, Username, Service_name, to)

ACP		*Acp;		/* Handle for ACP communication */
UINT32		logid,		/* sequence number for logging */
		linet;		 /* Annex Internet address */
int		port,		/* Annex port number */
		service;	/* SERVICE_CONNECT */
char		*Username,	/* user associated with port */
		*Service_name;	/* LAT service name */

struct sockaddr_in *to;	  /* who sent the message we will respond to */
{
    char	String[64+10];

    /*  generic acknowledge - return to remote caller  */

    (void)acp_acknowledge(Acp, to);

    /*  log a message for accounting/security purposes  */

    (void)sprintf(String, "%s:%s", Service_name, Username);

    log_message(linet, logid, port, service, EVENT_LOGOUT, String);

    /*  terminate this session, which exit()s  */

    terminate_session();

    /*  dummy return  */

    return;
}


/*
 *	net_to_port_logout()
 *
 *	Called when Annex closes a previously secured port server connection.
 */

net_to_port_logout(Acp, logid, linet, port, service, rinet, Username, to)

ACP		*Acp;			/* Handle for ACP library */
UINT32		logid,			   /* log sequence number */
		linet,			/* Annex Internet address */
		rinet;			/* remote host's Internet */
int		port,			 /* physical/virtual port */
		service;		 /* service SERVICE_PORTS */
char		*Username;		/* user associated w/port */
struct sockaddr_in *to;		/* who sent the message we will respond to */
{
	char	String[64];
	char	Hostname[32];

	/*  generic acknowledge - return to remote caller  */

	(void)acp_acknowledge(Acp, to);

	/*  log a message for accounting/security purposes  */

	inet_name(Hostname, rinet);

#if	(USER_VALIDATION)

	(void)sprintf(String, "%s:%s", Hostname, Username);

#else

	(void)sprintf(String, "%s:%s", Hostname, unknown);

#endif

	log_message(linet, logid, port, service, EVENT_LOGOUT, String);

	/*  terminate this session, which exit()s  */

	terminate_session();

	/*  dummy return  */

	return;
}

/*
 * 	appletalk_profile
 *
 *	This function is called by the annex when a user starts an
 * 	ARAP or ATCP/PPP session on the annex.  
 */

appletalk_profile(Acp, logid, inet, port, service, Name, nve, to)

ACP		*Acp;		/* Handle to pass to library functions */
UINT32		logid,				/* Log sequence number */
		inet;			     /* Annex Internet address */
int		port,		       /* physical/virtual port number */
		nve,			/* tell me nve details 	       */
		service;	   	      /* Expect SERVICE_DIALUP */
ACP_STRING	Name;					   /* Username */
struct sockaddr_in *to;	    /* who sent the message we will respond to */
{
	int connect_time;
        int status, error, guest;
        Uprof up;
	
        guest = !(strcmp("<Guest>",Name));
        error = get_user_profile(Name, &up);
        if (error == ACPU_ESUCCESS) {

		if (	(up.up_at.at_zone_combined != 0)	||
			(up.up_at.at_zones != 0)		||
			(strlen(up.up_at.at_zonelist) != 0)	||
			(strlen(up.up_at.at_passwd) != 0)	||
			(up.up_at.at_connect_time != 0)		||
			(up.up_at.at_nve_exclude != 0)		||
			(strlen(up.up_at.at_nve) != 0)	)
		{
	  	return_appletalk_profile(Acp,to, REQ_GRANTED, 
			           up.up_at.at_zone_combined, 
			           up.up_at.at_zones, 
				   up.up_at.at_zonelist, 
                                   up.up_at.at_passwd, 
                                   up.up_at.at_connect_time, 
				   up.up_at.at_nve_exclude, 
				   nve ? up.up_at.at_nve : 0);

	  	log_message(inet, logid, port, service, EVENT_LOGIN, Name);
		} else {
		    return_appletalk_profile(Acp,to,REQ_DENIED, 0,0,0,0,0,0,0);
		    log_message(inet, logid, port,service,EVENT_NOPROVIDE,Name);
        	}

	} 
        else if (error == ACPU_ENOUSER && guest) {
	  return_appletalk_profile(Acp,to,REQ_GRANTED, 0, 0, 0, 0, 0,0,0);
	  log_message(inet, logid, port, service, EVENT_LOGIN, Name);
	}
        else {
	  return_appletalk_profile(Acp,to,REQ_DENIED, 0, 0, 0, 0, 0, 0,0 );
	  log_message(inet, logid, port, service, EVENT_NOPROVIDE, Name);
        }
}

int
hook_callback(Acp,logid,code,cmnd)
ACP *Acp;		/* Handle to pass to library functions */
UINT32 logid;
int code;
char *cmnd;
{
#ifdef ALLOW_HOOKS
	FILE *fp;
	char buf[256],*cp;
	int i;

/*
 * This call permits ACP to print back to the user.  The "pending" code
 * tells the Annex to wait for hook_callback_string instead, and to
 * process any outputstring or promptstring requests in the meantime.
 */
	return_hook_callback(Acp,HCB_PENDING,NULL);

#ifdef ALLOW_HOOKS_DEMO
/* Check for the added "rls" command here */
	if (code == HCB_BAD_COMMAND && strncmp(cmnd,"rls",3) == 0 &&
	    (cmnd[3] == '\0' || cmnd[3] == ' ')) {

	/* Build a command line for the system here. */
		strcpy(buf,"/bin/ls");
		if (cmnd[3] == ' ')
			strcat(buf,cmnd+3);
		if ((cp = strpbrk(buf,";>`&")) != NULL)
			*cp = '\0';	/* avoid security problems */

	/* Start the "ls" command and echo the data back to the user */
		fp = popen(buf,"r");
		if (fp == NULL)
			i = outputstring(Acp,"Error executing ",NULL);
		else
			i = outputstring(Acp,"Executing ",NULL);
		strcat(buf,"\n");
		if (i == 0)
			i = outputstring(Acp,buf,NULL);
		if (i == 0 && fp != NULL) {
			for (;;) {
				/* Read in some text from the "ls" */
				i = fread(buf,sizeof(*buf),
					sizeof(buf)-1,fp);
				if (i <= 0)
					break;
				buf[i] = '\0';
				/* Write it out to the user */
				i = outputstring(Acp,buf,NULL);
				if (i != 0)
					break;
			}
			pclose(fp);
			i = outputstring(Acp,"/bin/ls done.\n",NULL);
		}
/*
 * Delete the user's input so that the CLI doesn't attempt to execute
 * the "rls" command.  We could have replaced this with a different
 * command here ...
 */
		cmnd[0] = '\0';
	}

#endif /* ALLOW_HOOKS_DEMO */

/*
 * Check for a prompting CLI here with a predefined macro.  This happens
 * when the "library" request is entered in this example.  We return the
 * preprogrammed "telnet -s library" string to the CLI the first time,
 * and then set the command to "hangup -q" so that we'll exit after the
 * telnet is done.  If we wanted to hand control of the CLI back to the
 * user, we could have returned HCB_TERMINATE in hook_callback_string.
 */
	if (code == HCB_BEFORE_PROMPT && stuff_command[0] != '\0') {
		strcpy(cmnd,stuff_command);
		strcpy(stuff_command,"hangup -q");
	}

/*
 * Return the possibly modified command to the CLI.  This call frees up
 * the CLI and restarts execution on the Annex.
 */
	hook_callback_string(Acp,HCB_RETURN,cmnd);
#else
	/* Default:  just return the command to the CLI */
	return_hook_callback(Acp,HCB_RETURN,cmnd);
#endif /* ALLOW_HOOKS */
}


/*
 *	dialup_address_validate()
 *
 *	Look up username in (INSTALL_DIR/acp_dialup) file. 
 *	Return local and remote addresses associated with first 
 *	instance of this username and Annex.
 *	
 */
int
dialup_address_validate(User, inet_addr, loc, rem, port)

char	*User;
UINT32	inet_addr, *loc, *rem;
int port;
{
	char	*name ;
	UINT32  inet, local=0, remote=0;
	int	found_match = 0;
	int	errorcode,i = 0;

	if (setacpdialup() < 0)
		return 0;

#ifdef DEBUG_DIALUP
	debug_dialup = 1;
#endif

	if (debug_dialup) {
	  LINE;
	  printf("Data from Annex:\nname: %-10s   inet: %8x    port: %d\n", 
				User, inet_addr,port);
	  LINE;
	  printf("Data from acp_dialup file:\n"); 
	  fflush(stdout);  
	}


		errorcode = findacpdialup(&name, &inet, &local, &remote,
					port, User, inet_addr);

		if (errorcode == -1) {
		  if (debug_dialup) {
		    printf("-----  Reached end of acp_dialup file.  -----\n");
		    LINE;
		  }
		  goto goback;
		}

		/*
		 * If we got a valid entry from getacpdialup() and
		 * the username and inet addresses match, then
		 * give the corresponding local and remote addresses
		 * and return.
		 */

		if (errorcode == 1) {
		    if (debug_dialup) {
			printf("\nname:%-10s    inet:%8x    loc:%8x    rem:%8x",
				name,inet,local,remote);
			  LINE;
			  fflush(stdout);  
		    }
		    if (found_match =  
		  	(!strncmp(User, name, ACP_MAXSTRING)  &&
			( inet == 0 || inet == inet_addr))) {
			*loc = local == 0 ? htonl(inet_addr) : htonl(local);
			*rem = htonl(remote);
		    }
		}
goback:
	endacpdialup();
	return found_match;
}


/*
 *	acp_validate()
 *
 *	Look up name and password combination in an /etc/passwd compatible
 *	password file (INSTALL_DIR/acp_passwd).  If several passwords exist
 *	for the same username, any valid password is accepted.  Null entries
 *	and "*" entries (disabled) for passwords are not considered valid.
 *
 *	Additionally, for systems using a shadow file (/etc/shadow or
 *	INSTALL_DIR/acp_shadow), this file is checked for a valid password
 *	and the appropriate aging is performed if at least one password entry
 *	in the /etc/passwd file is "x".
 *
 *	Return codes:
 *		0 - invalid username/password (or account expired)
 *		1 - valid username/password
 *		2 - valid username/password, but password to expire soon
 *		3 - valid username/password, but password has expired
 *		4 - valid username/password, but account to expire soon
 *
 *	In cases 2 and 4, the global variable "daysleft" is set to the number of
 *	days left until this event.
 */

int
acp_validate(User, Password)
char	*User, *Password;
{

/* Use SunOS 4.1.x adjunct file call - save a lot of work. */
#ifdef SUNADJUNCT
	int found_match;
	found_match = pwdauth(User, Password);
        if (found_match == 0) return 1; else return 0;
#else  /* SUNADJUNCT */

#ifndef KERBEROS
	struct	passwd *pwd;
	char	*pw;
	int	found_match = 0;

#ifdef USESHADOW
	struct	spwd *spwd;
	int	found_x = 0;
	INT32	today,expire;
#endif /* USESHADOW */

	setacppw();
	while ((pwd = getacppw()) != NULL) {
		if (pwd->pw_passwd[0] == '\0')
			continue;	/* Disallow null passwords */
		if (strncmp(pwd->pw_name, User, ACP_MAXSTRING) != 0)
			continue;
#ifdef USESHADOW
		if (strcmp("x",pwd->pw_passwd) == 0) {
			found_x = 1;
			continue;
		}
#endif /* USESHADOW */
		pw = crypt(Password, pwd->pw_passwd);
		if (strncmp(pw, pwd->pw_passwd, ACP_MAXSTRING) == 0) {
			found_match = 1;
			break;
		}
	}
	endacppw();

#ifdef USESHADOW
	if (found_match)
		return 1;
	if (!found_x)
		return 0;
	today = DAY_NOW;
	setacpsp();
	while ((spwd = getacpsp()) != NULL) {
		if (spwd->sp_pwdp[0] == '\0')
			continue;	/* Disallow null passwords */
		if (strncmp(spwd->sp_namp, User, ACP_MAXSTRING) != 0)
			continue;
		pw = crypt(Password, spwd->sp_pwdp);
		if (strncmp(pw, spwd->sp_pwdp, ACP_MAXSTRING) != 0)
			continue;
	/* Check for an expired account */
		if (spwd->sp_expire > 0 && today > spwd->sp_expire)
			break;
/*
 * We're not implementing sp_inact here to check for inactive accounts.
 * To do this, the time/date stamp in /usr/adm/wtmp should be checked
 * and validated.  This would probably also require writing to this
 * file to update this stamp after the user logs into the Annex.
 */
	/* Calculate password expiration date */
		if (spwd->sp_lstchg > 0 && spwd->sp_max > 0)
			expire = spwd->sp_lstchg+spwd->sp_max;
		else
			expire = 0;
	/* Check for an expired password */
		if (today < spwd->sp_lstchg || (expire > 0 && today > expire)) {
			strncpy(old_password,spwd->sp_pwdp,
				sizeof(old_password));
			found_match = 3;
	/* Check for an account that is soon to expire */
		} else if (spwd->sp_expire > 0 &&
		    today >= spwd->sp_expire-spwd->sp_warn) {
			found_match = 4;
			daysleft = spwd->sp_expire-today;
	/* Check for a password that is soon to expire */
		} else if (expire > 0 &&
		    today >= expire-spwd->sp_warn) {
			found_match = 2;
			daysleft = expire-today;
		} else
			found_match = 1;
		break;
	}
	endacpsp();
#endif /* USESHADOW */

	return found_match;

#else 	/* KERBEROS */
    char tktfile[MAXPATHLEN];
    char krbrlm[REALM_SZ], uname[ANAME_SZ];
    int krbval;
	     
 
    if (krb_get_lrealm(krbrlm, 1) != KSUCCESS) {
    	(void)strncpy(krbrlm, KRB_REALM, sizeof(krbrlm));
    }
		
    /* Use our pid for the ticket file name; it goes
       away immediately, anyway */
    (void)sprintf(tktfile,"%s%d", KRB_TK_DIR, getpid());
    krb_set_tkt_string(tktfile);
		    
    strcpy(uname, User);
     
    krbval = krb_get_pw_in_tkt(uname, "", krbrlm, "krbtgt",
				krbrlm, DEFAULT_TKT_LIFE, Password);
    bzero(Password, strlen(Password));
    dest_tkt();
	 
    switch(krbval) {
	case INTK_OK:
      	    if (debug)
	      	printf("Krb-auth worked for user %s\n", User);
	    return(1);
	default:
	    if (debug)
        	printf("Krb-auth FAILED for user %s (%s)\n", User,
		       		krb_err_txt[krbval]);
	    return(0);
    }
#endif /* KERBEROS */

#endif /* SUNADJUNCT */

}


#ifdef USESHADOW

/*
 * This is called by routines that use acp_validate.  It takes appropriate
 * actions based on the returned code.
 */

int
warn_user(Acp,validp,user,to)
ACP *Acp;		/* Handle to pass to library functions */
int *validp;
char *user;
struct sockaddr_in *to;	/* who sent the message we will respond to */
{
    ACP_LSTRING temp,temp2;
    char *cp;
    int i,vcount;

    switch (*validp) {
    case 2:		/* Password aging warning */
	if (daysleft <= 0)
	    strcpy(temp,ACP_WARNINGT);
	else if (daysleft == 1)
	    strcpy(temp,ACP_WARNINGM);
	else
	    sprintf(temp,ACP_WARNING,daysleft);
	break;
    case 3:		/* Password has expired -- prompt for new one */
	*validp = 0;	/* assume that this won't work */
	if (user == NULL || user[0] == '\0')
	    return -1;
/* "vcount" allows user to enter "illegal" passwords if he's persistent. */
	vcount = 0;
	if ((i = outputstring(Acp, ACP_EXPIRED, to)) != 0)
	    return i;
	for (;;) {
	    if (!promptstring(Acp,temp,ACP_NEWPASS,0,INPUT_TIMEOUT,to))
		return -1;
	    if (temp[0] == '\0')
		return -1;
	    if ((cp = test_password(temp)) != NULL && vcount < 3) {
		if ((i = outputstring(Acp, cp, to)) != 0)
		    return i;
		vcount++;
		continue;
	    }
	    vcount = 0;
	    if (!promptstring(Acp,temp2,ACP_NEWPASS2,0,INPUT_TIMEOUT,to))
		return -1;
	    if (strcmp(temp,temp2) != 0) {
		if ((i = outputstring(Acp, ACP_PASSMATCH, to)) != 0)
		    return i;
	    } else
		break;
	}
	bzero(temp2,sizeof(temp2));
	cp = change_password(user,old_password,temp);
	if (cp != NULL) {
	    if (*cp == ' ')
		strcpy(temp,cp+1);
	    else
		sprintf(temp,"%s: %s.\n",cp,sys_errlist[errno]);
	    break;
	}
	*validp = 1;
	return 0;
    case 4:		/* Account aging warning */
	if (daysleft <= 0)
	    strcpy(temp,ACP_AWARNINGT);
	else if (daysleft == 1)
	    strcpy(temp,ACP_AWARNINGM);
	else
	    sprintf(temp,ACP_AWARNING,daysleft);
	break;
    default:
	return 0;
    }
    return outputstring(Acp, temp, to);
}

#endif /* USESHADOW */


#ifdef SECURID_CARD

/*
 *	acp_securid_validate()
 *
 *	Look up name and password combination by calling the Security
 *	Dynamics Ace/Server utility.  
 */

acp_securid_validate(User, Password, Acp, to)

char	*User, *Password;
ACP	*Acp;			/* Handle to pass to library calls */
struct  sockaddr_in *to;	/* who sent the message we will respond to */
{
	int	found_match = 0, got_str = 0, i;
	struct 	SD_CLIENT sd_dat, *sd;
	ACP_LSTRING nextcode, newpin, pinprompt;
	char	*s;
	char	*pintype, pinsize[20];
	short	flag;

	if (debug)
	    printf ("acp_securid_validate\n");

	sd = &sd_dat;
	memset(sd, 0, sizeof(*sd));
	nextcode[0] = newpin[0] = '\0';

#ifdef ACE1_2
	creadcfg();
	if (sd_init (&sd))
	    {
	    if (debug)
		printf ("failed to init client-server communications\n");
	    return found_match;
	    }
#endif
	switch (sd_check (Password, User, sd))
	    {
	    case ACM_OK:
		found_match = 1;
		break;

	    case ACM_NEXT_CODE_REQUIRED:
		got_str = promptstring(Acp, nextcode, SID_NEXTCODEPROMPT, 0,
				 INPUT_TIMEOUT + 60, to);
		if (got_str < 0)
		    break;
		switch (sd_next(nextcode, sd))
		    {
		    case ACM_OK:
			found_match = 1;
			break;

		    case ACM_ACCESS_DENIED:
		    case ACM_NEXT_CODE_BAD:
		    default:
			break;
		    }
	        break;
	    case ACM_NEW_PIN_REQUIRED:
		pintype = (sd->alphanumeric) ? SID_PINCHAR : SID_PINDIGIT;
		if (sd->min_pin_len == sd->max_pin_len)
		    sprintf (pinsize, SID_PINSIZE, sd->min_pin_len);
		else
		    sprintf (pinsize, SID_PINSZRANGE, sd->min_pin_len,
				sd->max_pin_len);
		if (sd->user_selectable) {
		    sprintf (pinprompt, SID_NEWPINPROMPT, pinsize, pintype);
		    outputstring (Acp, pinprompt, to);
		    outputstring (Acp, SID_OR, to);
		}
		outputstring (Acp, SID_NEWPIN_2, to);
		outputstring (Acp, SID_OR, to);
		outputstring (Acp, SID_NEWPIN_3, to);
		/* include terminator in input string */
		/* in order to distinguish between a  */
		/* timeout and a <return>.            */
		flag = GET_TERMINATOR;
		got_str = promptstring_wt(Acp, newpin, " ",
				0, INPUT_TIMEOUT, to, &flag);
		if (got_str < 0)
		    /* timeout */
		    break;
		/* delete terminator from string (unless old annex)*/
		if (flag == GET_TERMINATOR)
		    {
		    if (got_str == 0)
			/* timeout */
			break;
		    newpin[strlen(newpin) -  1] = '\0';
		    }

		if (debug) {
		    s = &newpin[0];
		    printf ("---- securid_validate ---- newpin: %s\n", newpin);
		    for (i = 0; i < strlen(newpin) + 1; i++) {
			printf ("%02x ", s[i]);
			if (( (i + 1) % 16) == 0)
			    printf ("\n");
		    }
		    printf ("\n");
		}
		    
		if (newpin[0] == 0x04 && strlen(newpin) == 1) {
		    /* got a control-D */
		    sd_pin ("", CANCELLED, sd);
		    break;
		}
		else if (strlen(newpin) == 0) {
		    /* got a <return> */
		    sprintf (pinprompt, SID_SYSGENPIN, sd->system_pin);
		    outputstring(Acp, pinprompt, to);
		    if (sd_pin(sd->system_pin, 0, sd) != ACM_NEW_PIN_ACCEPTED) {
			break;
		    }
		}
		else if (sd->user_selectable) {
		    /* got a new pin */
		    strcpy (sd->system_pin, newpin);
		    newpin[0] = '\0';
		    got_str = promptstring(Acp, newpin, SID_PINREENTRY, 0,
				INPUT_TIMEOUT, to);
		    if(strcmp (sd->system_pin,newpin) != 0) {
			sd_pin ("", CANCELLED, sd);
			break;
		    }
		    if (sd_pin(sd->system_pin, 0, sd) != ACM_NEW_PIN_ACCEPTED) {
			break;
		    }
	 	}
		outputstring(Acp, SID_LOGNEWPIN_1, to);
		outputstring(Acp, SID_LOGNEWPIN_2, to);
		nextcode[0] = '\0';
		got_str = promptstring(Acp, nextcode, SID_PASSPROMPT, 0,
             		INPUT_TIMEOUT + 60, to);
		if (got_str < 0)
		    break;
		if (sd_check (nextcode, User, sd) == ACM_OK)
		    found_match = 1;
		break;

	    default:
	    case ACM_ACCESS_DENIED:
		break;

	    }

	return found_match;
}

#endif /* SECURID_CARD */



/*
 *	available()
 *
 *	Is given Annex (linet) allowed to connect to the given Host (rinet)
 *	according to the entries in INSTALL_DIR/acp_restrict?
 *
 *	See annex_to_net() function for details and examples.
 */

#define END_OF_FILE	1001
#define IS_ANNEX	1002
#define IS_ANNEX_NOT	1003
#define IS_HOST		1004
#define SIZE_INET	64
#define EXCLUDE_LIST	1
#define INCLUDE_LIST	2

available(linet, rinet)

UINT32		linet, rinet;	/* Annex in question, remote host requested */
{
	int	done = 0;
	int	got_annex = 0;
	int	status;
	char	token[SIZE_INET];

	/*
	 *  Scan file until EOF, or until a match is encountered.  In the
	 *  event of no match (EOF), take default action - allow access.
	 */

	while(!done)
	{
	    status = get_string(token);

	    switch(status)
	    {
		case END_OF_FILE:

		    done = -1;
		    break;

		case IS_ANNEX:

		    if(wild_match(token, linet))
			got_annex = EXCLUDE_LIST;
		    else
			got_annex = 0;
		    break;

		case IS_ANNEX_NOT:

		    if(wild_match(token, linet))
			got_annex = INCLUDE_LIST;
		    else
			got_annex = 0;
		    break;

		case IS_HOST:

		    if(got_annex && wild_match(token, rinet))
			if(got_annex == EXCLUDE_LIST)
			    return(0);
			else
			    return(1);

		default:

		    break;
	    }
	}

	/*  End of file - no explicit restriction found - include me  */

	return(1);
}


/*
 *	get_string()	- low level parser for acp_restrict file
 */


get_string(String)

char	*String;
{
	static	int	start_file = -1, end_file = 0;
	static	FILE	*rfile;

	char		restrict_pth[PATHSZ];
	int		slen = 0,
			value;

	ACP_RESTRICT(restrict_pth);
	if(start_file)
	  if(!(rfile = fopen(restrict_pth, "r")))
	    return END_OF_FILE;

	start_file = 0;

	while(!end_file)
	{
	    value = getc(rfile);

	    switch(value)
	    {
		case EOF:

		    end_file = -1;
	
		case ' ':
		case '\t':
		case '\r':
		case '\n':
		case '\f':
		case '\0':
		case ',':
		case '\\':

		    if(slen)
		    {
			String[slen] = '\0';
			return IS_HOST;
		    }
		    break;

		case ':':

		    if(slen)
		    {
			String[slen] = '\0';
			return IS_ANNEX;
		    }
		    break;

		case '~':

		    if(slen)
		    {
			String[slen] = '\0';
			return IS_ANNEX_NOT;
		    }
		    break;


		default:

		    if(slen < (SIZE_INET - 1))
		      String[slen++] = value;
	    }
	}
	(void)fclose(rfile);
	return END_OF_FILE;
}


/*
 *	wild_match()	- does host identified by string match internet?
 *
 *	Wildcards, names, or Internet addresses in dot notation are allowed.
 *	A wildcard may be a * in any position of an Internet address.  It
 *	is usually used to specify every host on a network (net.*) or every
 *	host on all networks (*).  For example:
 *
 *	*		matches any host
 *	60.*		matches any host on network 60, equivalent to 60.*.*.*
 *	131.1.*		matches any host on net 131.1, equivalent to 131.1.*.*
 *	195.5.35.*	matches any host on network 195.5.35
 *	66.*.1.*	matches 66.0.1.1, 66.1.1.0, 66.2.1.25, etc.
 */

wild_match(token, internet)

char		*token;				/* hostname or dot notation */
UINT32		internet;			/* inet addr, internal fmt */
{
	UINT32		inetwild;		/* addr derived from token */
	UINT32		maskwild;		/* mask derived from token */

	unsigned char	*pmask = (unsigned char *)&maskwild;
	int		mx, tx;			/* index into mask, token */

	/*
	 *  If not a digit or '*' in the first position, assume hostname
	 *  Return match based on hostname translation matching internet
	 */

	if(token[0] != '*' && !isdigit(token[0]))
		return inet_match(token,internet,0);
	/*
	 *  Otherwise, we must create a possible wildcard mask (don't care)
	 *  and replace any *'s in the token with ASCII 0's.
	 */

	for(tx = mx = 0 ;; tx++)			/* breaks on EOS */
	{
		if(token[tx] == '.' || token[tx] == 0)	/* dot or null? */
		{
			if(token[tx - 1] == '*')	/* wildcard part? */
			{
				token[tx - 1] = '0';	/* change * to 0 */
				pmask[mx] = 0xff;
			}
			else
				pmask[mx] = 0;		/* else normal part */

			if(token[tx] == 0)		/* end of string? */
				break;			/* end loop */
			else
				mx++;			/* else bump */
		}
	}

	/*
	 *  Now we adjust for incomplete (not four part) addresses
	 *  Take character at current index, and propagate it
	 */

	while(mx < 3)		/* while index (# of dots) is not at end */
	{
		pmask[mx + 1] = pmask[mx];	/* next byte equals current */
		mx++;				/* avoid confusion */
	}
	
	/*
	 *  Translate token to internet address, mask and match w/internet
	 */
	return inet_match(token,internet,maskwild);
}


/*
 *  secure_cache()
 *
 *  Create internal tables for annex encryption keys.
 *  A separate table is created for wildcard entries.
 *  Bytes containing nulls match corresponding byte of internet address.
 *
 *  This routine is invoked at initialization time, or by a SIGHUP.
 */

#define GOT_END		1001
#define GOT_ANNEX	1002
#define LAST_ANNEX	1003
#define GOT_PASSWORD	1004
#define SIZE_TOKEN	64
#define ANNEX_LIST	2501
#define	ANNEX_PASSWORD	2502

secure_cache()
{
	int		done = 0;		/* set on end of file */
	int		status;			/* return from get_token() */
	int		wildcard;		/* is token a wildcard? */
	int		i;			/* use me for loops */
	int		wildmark = 0;		/* mark wildcard cache */
	int		tamemark = 0;		/* mark regular cache */
	int		quest;			/* which token is expected? */
	UINT32		address;		/* internet address */
	KEYDATA		*key;			/* pointer to crypt table */
	char		token[SIZE_TOKEN + 1];	/* token from get_token() */
	FILE 		*sfile;			/* file pointer */
	char		keypath[PATHSZ];

	/*
	 *  Free any previously allocated KEYDATA tables, since this
	 *  routine is also called on a SIGHUP and tables are replaced
	 */

	for(i = 0; i < Nmalloced; i++)
	    free(Allocated[i]);

	Nwild = Ntame = Nmalloced = 0;

	ACP_KEYS(keypath);
	sfile = fopen(keypath, "r");

	if(!sfile)
	    done = -1;

	quest = ANNEX_LIST;

	while(!done)
	{
	    status = get_token(sfile, quest, token, &wildcard);

	    switch(status)
	    {
		case GOT_END:

		    done = -1;
		    break;

		case LAST_ANNEX:

		    quest = ANNEX_PASSWORD;

		case GOT_ANNEX:

		    if(!strlen(token))
			address = 0;
		    else
		        address = inet_address(token);

		    if(wildcard)
		    {
			if(Nwild < MAX_WILD)
			    Wild[Nwild++].addr = address;
		    }
		    else if(address)
		    {
			if(Ntame < MAX_TAME)
			    Tame[Ntame++].addr = address;
		    }
		    break;

		case GOT_PASSWORD:

		    token[KEYSZ - 1] = '\0';  /* limit size */

		    if(Nmalloced < MAX_KEYS)
		    {
			key = make_table(token, (KEYDATA *)0);

			Allocated[Nmalloced++] = (char *)key;

			while(wildmark < Nwild)
			    Wild[wildmark++].ckey = key;

			while(tamemark < Ntame)
			    Tame[tamemark++].ckey = key;
		    }
		    quest = ANNEX_LIST;
		    break;

		default:

		    break;
	    }
	}

	if(debug)
	{
		printf("Wild entries:\n");
		    for(i = 0; i < Nwild; i++)
			printf("Address %8.8x, key <%s>\n",
			    Wild[i].addr, Wild[i].ckey->password);

		printf("Tame entries:\n");
		for(i = 0; i < Ntame; i++)
		    printf("Address %8.8x, key <%s>\n",
			    Tame[i].addr, Tame[i].ckey->password);
	}
	if(sfile)
	    (void)fclose(sfile);
	return;
}


/*
 *	get_token()	- low level parser for acp_keys file
 */

get_token(file, quest, String, wild)

FILE	*file;		/* file pointer for ACP_KEYS file */
int	quest;		/* quest is for an Annex address or a password? */
char	*String;	/* token - Annex identifier or password */
int	*wild;		/* is this token a wildcard entry? */
{
	int		slen = 0,	/* length of current token */
			endoffile = 0,	/* reached end of file */
			value;		/* returned character or EOF */

	*wild = 0;
	String[0] = '\0';

	while(!endoffile)
	{
	    value = getc(file);

	    switch(value)
	    {
		case EOF:

		    endoffile = -1;
	
		case ' ':
		case '\t':

		    break;				/* skip white space */

		case '\\':

		    value = getc(file);

		    if(value == EOF)			/* cant escape EOF */
			endoffile = -1;
		    else if(value == '\n')		/* ignore newlines */
			break;
		    else goto data;			/* else it is data */

		case '#':

		    /* skip from # to end of line */

		    while((value = getc(file)) != '\n' && value != EOF)
			;

		case '\n':

		    String[slen] = '\0';		/* can be null */
		    return GOT_PASSWORD;		/* got the password */

		case ':':

		    if(quest == ANNEX_PASSWORD)
			goto data;

		    String[slen] = '\0';		/* null terminate */
		    return LAST_ANNEX;			/* end of list */

		case ',':
		case ';':

		    if(quest == ANNEX_PASSWORD)		/* can be in passwd */
			goto data;

		    String[slen] = '\0';		/* null terminate */
		    return GOT_ANNEX;			/* got an Annex */

		case '*':

		    *wild = -1;				/* got a wildcard */
		    value = '0';			/* treat as zero */

data:
		default:

		    if(slen < (SIZE_TOKEN - 1))		/* token character */
			String[slen++] = value;
	    }
	}
	return GOT_END;
}


/*
 *  annex_key()
 *
 *  Retrieve key from internal address/key cache.
 *  This information is derived from ACP_KEYS file, at start or SIGHUP.
 */

KEYDATA *
annex_key(iaddr)

UINT32		iaddr;			/* internet address of an Annex */
{
	int		i, j;		/* indices for loops */
	char		*x, y[4];	/* point to entry, copy of iaddr */

	/* search exact match table first, then search wildcard table */

	for(i = 0; i < Ntame; i++)	/* straight comparison */
	{
		if(Tame[i].addr == iaddr)
		    return Tame[i].ckey;
	}
	for(i = 0; i < Nwild; i++)	/* wildcard comparison */
	{
		x = (char *)&Wild[i].addr;
		bcopy((char *)&iaddr, y, sizeof(iaddr));
		for(j = 0; j < sizeof(iaddr); j++)
		    if(x[j] == '\0')
			y[j] = '\0';

		if(Wild[i].addr == *(UINT32	 *)y)
		    return Wild[i].ckey;
	}
	return (KEYDATA *)0;
}

#ifdef PORT_PASSWORD

/*
 * Look for a special port password for this port.
 * These are stored in the acp_passwd file as inetaddr.port, i.e.,:
 * for milo (132.245.1.200) port 13:
 *	132.245.1.200.13:xxxxxxxxxxxxx:
 */
void
acp_special(Special, inet, port)

char		*Special;
UINT32		inet;
int		port;
{
	unsigned char	*Pinet = (unsigned char *)&inet;

	/*
	 *  Convert internet address and port number to special string
	 */

	(void)sprintf(Special, "%u.%u.%u.%u.%u",
			Pinet[0], Pinet[1], Pinet[2], Pinet[3], port);
	return;
}

acp_getusr(Name)

char	*Name;
{
	struct	passwd *pwd;

	/*
	 *  Check password file for an entry
	 */
	setacppw();
	while(pwd = getacppw()) {
		if(!strcmp(pwd->pw_name, Name))
			return -1;
	}
	endacppw();

	return 0;	/* no entry found */
}

#endif /* PORT_PASSWORD */

/*
 * check_port_pool() returns TRUE if the port/Annex pair belongs to
 * "pool_name" as defined in the file acp_userinfo.
 * When no "pool_name" is specified all pools are checked.
 * returns DIAL_SUCC if pool entry was found.
 */
int
check_port_pool(annex, port, pool_name)
UINT32		annex;
int		port;
char		*pool_name;
{
int	error;

if(debug)
	{
	if(pool_name != 0)
	printf("annex=0x%x, port=%d, pool_name=%s.\n",annex, port, pool_name);
	else
	printf("annex=0x%x, port=%d, pool_name=0.\n",annex, port);
	}

error = get_pool_entry_by_addr(pool_name, annex, port);

if(debug)
	printf("check_port_pool: error=%d\n",error);

return(error);
}

int
strnumcmp(s1, s2)
	 char *s1, *s2;
{
  while (TRUE) {
	while (*s2 && ((*s2 <= '0') || (*s2 >= '9')))
	  s2++;
	while (*s1 && ((*s1 <= '0') || (*s1 >= '9')))
	  s1++;
	if (*s1 == *s2++) {
	  if (*s1++ == '\0')
		return(0);
	}
	else
	  return (*s1 - *--s2);
  }
}
	
/*
 * access_code_validate() scans the acp_userinfo file and fills in
 * the user profile arguments: the telephone number, the port mask,
 * the inbound and the outbound modem pool names.
 * returns TRUE if username/accesscode was found, FALSE otherwise.
 */
int
access_code_validate(Name, Access_code, Phone, Job,In_pool, Out_pool, ipx_phone)
char	*Name;
char	*Access_code;
char	*Phone;
char	*Job;
char	*In_pool;
char	*Out_pool;
char	*ipx_phone;
{
int	error;
Access	ac_info;
int	len, index, phone_index;
char	*phone_str = ""; 
struct  _phone *_phone_ptr;

  *Phone = '\0';

  if (debug) {
	printf("access_code_validate: name=%s acc=%s\n",
		Name, Access_code);
	if (ipx_phone) {
		printf(" ipx_phone = %s", ipx_phone);
	}
	printf("\n");
  }

  /* search the database */
  error = get_user_access(Name, Access_code, &ac_info);

  if (debug) {
	printf("get_user_access ret %d\n",error);
	if (error == DIAL_SUCC) {
		int i = 1;
		printf("... code:\t%s.\n",ac_info.ac_code);
        	printf("... phone #'s:");
        	for (_phone_ptr = ac_info.ac_phone_list; 
			_phone_ptr; _phone_ptr = _phone_ptr->next,i++) {
			if (((i % 3) == 1) && (i != 1))
				printf("\n\t"); 
			printf("\t\"%s\"  ", _phone_ptr->ac_phone);
		}
		printf("\n");
		printf("... inpool:\t%s.\n",ac_info.ac_inpool);
		printf("... outpool:\t%s.\n",ac_info.ac_outpool);
		printf("... job:\t%s.\n",ac_info.ac_job.j_string);
	}
  }

  if (error != DIAL_SUCC)
	return (FALSE);

  if (ipx_phone) {

	/*
	 * This is for IPX.
	 */

	int i;
	len = strlen(ipx_phone);
	if (index = ((len <= 2) ? 1 : 0))
		phone_index = atoi(ipx_phone);

	if (debug)
		printf("ipx_phone = %s    index = %d\n", 
			ipx_phone, index);

	for (i = 0, _phone_ptr = ac_info.ac_phone_list; 
		(!*phone_str) && _phone_ptr; 
		_phone_ptr = _phone_ptr->next) {

		if (index) {
			if (++i == phone_index) {
				phone_str = _phone_ptr->ac_phone;
			}
			continue;
		}

		if (!(strcmp(_phone_ptr->ac_phone, 
			IPX_CHARGE_BACK_TOK))) {
			phone_str = ipx_phone;
			continue;	
		}
		else if (!strnumcmp(ipx_phone, _phone_ptr->ac_phone))
		  phone_str = _phone_ptr->ac_phone;
	}
  }
  else {

	/*
	 * This is for CLI.
	 */

        if (ac_info.ac_phone_list)
                phone_str = ac_info.ac_phone_list->ac_phone;
  }

  /*
   * IPX accesscode entries must have a phone number.
   */
  if (ipx_phone && !*phone_str)  {
	return (FALSE);
  }

  if (debug)
	printf("access_code_validate: obtained phone=%s\n",
		phone_str);

  (void)strncpy(Phone, phone_str, strlen(phone_str));
  (void)strcpy(Job, ac_info.ac_job.j_string);
  (void)strcpy(In_pool, ac_info.ac_inpool);
  (void)strcpy(Out_pool,ac_info.ac_outpool);

  return(TRUE);
}

/*
 * check_dialback() is called by port_to_annex() on the host to
 * determine if this is a valid dialback request, in which case
 * the user is prompted for the access code and (if all goes
 * well) the dialback request is started.
 *
 * This is what we are trying to do:
 *
 *
 *	go through the username and password authentication
 *	if (this Annex port number not included in any pool)
 *		{
 *		this is a direct connect
 *		}
 *	else
 *		{
 *		prompt for Access_Code
 *		if (bad access code)
 *			goto reject_request
 *		else if (port not member if inbound pool)
 *			goto reject_request
 *		else if (outbound pool name not specified)
 *			this is a direct connect
 *		else if (telephone number not specified)
 *			prompt user for telephone number
 *		if (telephone number not given)
 *			goto reject_request
 *		else
 *			dial back
 *		}
 *
 */
void
check_dialback(Acp, logid, inet, port, service, to, Name)

ACP		*Acp;		/* Handle to pass to library functions */
UINT32		logid,				/* Log sequence number */
		inet;			     /* Annex Internet address */
int		port,		       /* physical/virtual port number */
		service;			 /* Expect SERVICE_CLI */
struct sockaddr_in *to;		/* who sent the message we will respond to */
char		*Name;
{
ACP_LSTRING Access_code,Phone;
char	Job[LEN_JOB], Port_mask[LEN_PORT_MASK];
char	In_pool[ACP_LONGSTRING], Out_pool[ACP_LONGSTRING];
char	Message[ACP_MAXSTRING * 3 + 1];
int	retries, passed;
int	got_str;
UINT32	target_inet;
unsigned short dialb_port=0;
PoolEntry       pool_info;
int	error;
Uprof	up;

Message[0] = 0; /* empty string */

for (retries = 0; retries < RETRIES_MAX; retries++) {
	passed = FALSE;
	got_str = promptstring(Acp, Access_code, ACP_ACCESSCODEPROMPT, 0,
				INPUT_TIMEOUT, to);
	if (got_str < 0)
		continue;
	if (!got_str) {
		passed = FALSE;
		break;
	} else {
		passed = access_code_validate(Name, Access_code, Phone,
					Job, In_pool, Out_pool, 0);
	}

	if (passed)
		break;
	}
if  ( ! passed ) {
	(void)outputstring(Acp, ACP_PERMDENIED, to);
	port_to_annex_authorize(Acp, REQ_DENIED, 0xffffffff, Name, to);
	(void) strcat ( Message, (char *)Name ) ;
	(void) strcat ( Message, ",bad access code" );
	log_message(inet, logid, port, service, EVENT_REJECT, Message);
	if (debug) {
		printf("check_dialback log: %s.\n",Message); 
		fflush(stdout);  
		}
	}
else	{
	/* check if port belongs to inbound pool */
	if (check_port_pool(inet, port, In_pool) != DIAL_SUCC) {
		/* request came from non authorized port: reject it */
		(void)outputstring(Acp, ACP_PERMDENIED, to);
		port_to_annex_authorize(Acp, REQ_DENIED, 0xffffffff, Name, to);
		(void) strcat ( Message, (char *)Name ) ;
		(void) strcat ( Message, ",bad inbound port");
		log_message(inet, logid, port, service, EVENT_REJECT, Message);
		if (debug) {
			printf("check_dialback log: %s.\n",Message); 
			fflush(stdout);  
			}
		goto finished;
		}
	else if (strlen(Out_pool) == 0) {
		/* outbound poolname undefined: this is a direct connect */
		(void)outputstring(Acp, ACP_PERMGRANTD, to);
                error = get_user_profile(Name,&up);
                if ((error != ACPU_ESUCCESS) || (up.up_climask == 0)) 
		  	(void)port_to_annex_authorize(Acp, REQ_GRANTED,
CLI_MASK,Name,to);
		else
                	(void)port_to_annex_authorize(Acp, REQ_GRANTED, 
up.up_climask, Name,to);

#ifdef DIALUP_SLIP
		set_dialup_slip(Acp, inet, port, Name);
#endif
		log_message(inet, logid, port, service, EVENT_LOGIN, Name);
		if (debug) {
			printf("check_dialback log: %s.\n",Name); 
			fflush(stdout);  
			}
		goto finished;
		}
	else if (get_port_pool(Out_pool, &pool_info) != ACPU_ESUCCESS) {
		/* Unable to read pool information: probably an invalid */
		/* pool name, or data base problem: reject request      */
		(void)outputstring(Acp, ACP_PERMDENIED, to);
		port_to_annex_authorize(Acp, REQ_DENIED, 0xffffffff, Name, to);
		(void) strcat ( Message, (char *)Name ) ;
		(void) strcat ( Message, ",bad outbound poolname");
		log_message(inet, logid, port, service, EVENT_REJECT, Message);
		if (debug) {
			printf("check_dialback log: %s.\n",Message); 
			fflush(stdout);  
			}
		goto finished;
		}
	else if (strlen(Phone) == 0) {
		/* prompt user for telephone number */
		got_str = promptstring(Acp, Phone, ACP_PHONEPROMPT, 1,
					INPUT_TIMEOUT, to);
		if (debug) {
			printf("check_dialback: got_str=%d, phone=%s.\n",
					got_str,Phone); 
			fflush(stdout);  
			}
		}
	if (strlen(Phone) == 0)	{
		/* error: don't have the phone number */
		(void)outputstring(Acp, ACP_PERMDENIED, to);
		port_to_annex_authorize(Acp, REQ_DENIED, 0xffffffff, Name, to);
		(void) strcat ( Message, (char *)Name ) ;
		(void) strcat ( Message, ",no phone number");
		log_message(inet, logid, port, service, EVENT_REJECT, Message);
		if (debug) {
			printf("check_dialback log: %s.\n",Message); 
			fflush(stdout);  
			}
		}
	else	{
		/* everything looks good */
		(void)outputstring(Acp, ACP_DIALBACKGRANTD, to);
                error = get_user_profile(Name,&up);
                if ((error != ACPU_ESUCCESS) || (up.up_climask == 0))
			port_to_annex_authorize(Acp, REQ_DIALB_GRANT, CLI_MASK,Name,to);
                else
                        port_to_annex_authorize(Acp, REQ_DIALB_GRANT,
up.up_climask,Name,to);

		if (debug) {
			printf("check_dialback: calling acp_request_dialout\n");
			fflush(stdout);
			}

		Message[0] = 0;
		(void) strcat ( Message, (char *)Name ) ;
		(void) strcat ( Message, ",dialback request");
		log_message(inet, logid, port, service, EVENT_LOGIN, Message);
		if (debug) {
			printf("check_dialback log: %s.\n",Message); 
			fflush(stdout);  
			}

		errno = -1;

		/*
		 * give the requesting annex time to disconnect
		 * and release the port. This is necessary in case
		 * both the dial in and the dial out go through
		 * the same annex port: if we don't sleep the
		 * dialout process will find the port is busy
		 * disconnecting. Wait a few seconds.
		 */
		(void)sleep(DIALBACK_DELAY);

		do {
		    int errno2;

		    /* get Annex addr.and port set from pool member */
		    (void) bcopy(pool_info.pe_portmap, Port_mask,LEN_PORT_MASK);
		    target_inet = pool_info.pe_hostaddr;

		    /* send request to appropriate annex */
		    errno = acp_request_dialout(target_inet, Name, Access_code,
				Phone, Job, Port_mask, &dialb_port, service, 0);

		    if(debug) {
			printf("acp_request_dialout returned errno=%d\n",errno);
			fflush(stdout);  
			}
		    if (errno == DIAL_SUCC)
			break;

		    /* dialback request failed: try next pool member */
		    errno2 = get_next_pool_entry(&pool_info);
		    if (errno2 == ACPU_ESUCCESS)
			errno = errno2;
		    if (debug) {
			printf("get_next_pool_entry ret %d\n",errno2);
			fflush(stdout);
			}
		} while (errno == ACPU_ESUCCESS);

		 dialback_log(inet, port, Name, Phone, target_inet, logid, 
				dialb_port, service, Message);

		if (debug) {
			printf("check_dialback log: %s.\n",Message); 
			fflush(stdout);  
			}
		}
	}
finished:
  return;
}



void
ipx_check_dialback(Acp, logid, inet, port, service, to, Name, Netnum, ipx_phone)

ACP		*Acp;		/* Handle to pass to library functions */
UINT32		logid,				/* Log sequence number */
		inet;			     /* Annex Internet address */
int		port,		       /* physical/virtual port number */
		service;			 /* Expect SERVICE_CLI */
struct sockaddr_in *to;	    /* who sent the message we will respond to */
ACP_STRING	*Name;
int		Netnum;				/* IPX Network Number	*/
char		*ipx_phone;		    /* Phone number to match on */
{
char	Access_code[ACP_MAXLSTRING], Phone[LEN_PHONE];
char	Job[LEN_JOB], Port_mask[LEN_PORT_MASK];
char	In_pool[ACP_LONGSTRING], Out_pool[ACP_LONGSTRING];
char	Message[ACP_MAXSTRING * 3 + 1];
int	retries, passed;
int	got_str;
UINT32 target_inet;
unsigned short dialb_port=0;
PoolEntry	pool_info;
int             error;
Uprof           up;

Message[0] = 0; /* empty string */
strcpy(Access_code,IPX_ACCESS_CODE_TOK);
bzero(Phone, LEN_PHONE);
passed = access_code_validate(Name, Access_code, Phone,
		Job, In_pool, Out_pool, ipx_phone);

if (debug)
	printf("passed = %d\n", passed);
if  ( ! passed ) {
	return_serial_validate(Acp, REQ_DENIED, to);
	(void) strcat ( Message, (char *)Name ) ;
	(void) strcat ( Message, ",bad access code" );
	log_message(inet, logid, port, service, EVENT_REJECT, Message);
	if (debug) {
		printf("ipx_check_dialback log: %s.\n",Message); 
		fflush(stdout);  
		}
	}
else	{
        /* check if we were trying to do chargeback without a phone no. */
        if (!strcmp(Phone,"unknown")) {
                (void)return_serial_validate(Acp, REQ_GRANTED, to);
                log_message(inet, logid, port, service, EVENT_LOGIN, Name);
                if (debug) {
                        printf("ipx_check_dialback log: %s.\n",Name);
                        fflush(stdout);
                        }
                goto finished;
                }

	/* check if port belongs to inbound pool */
	if (check_port_pool(inet, port, In_pool) != DIAL_SUCC) {
		/* request came from non authorized port: reject it */
		return_serial_validate(Acp, REQ_DENIED, to);
		(void) strcat ( Message, (char *)Name ) ;
		(void) strcat ( Message, ",bad inbound port");
		log_message(inet, logid, port, service, EVENT_REJECT, Message);
		if (debug) {
			printf("ipx_check_dialback log: %s.\n",Message); 
			fflush(stdout);  
			}
		goto finished;
		}
	else if (strlen(Out_pool) == 0) {
		/* outbound poolname undefined: this is a direct connect */
                error = get_user_profile(Name,&up);
                if ((error != ACPU_ESUCCESS) || (up.up_climask == 0)) 
		  	(void)return_serial_validate(Acp, REQ_GRANTED,
					to);
		else
                	(void)return_serial_validate(Acp, REQ_GRANTED, 
					to);

		log_message(inet, logid, port, service, EVENT_LOGIN, Name);
		if (debug) {
			printf("ipx_check_dialback log: %s.\n",Name); 
			fflush(stdout);  
			}
		goto finished;
		}
	else if (get_port_pool(Out_pool, &pool_info) != ACPU_ESUCCESS) {
		/* Unable to read pool information: probably an invalid */
		/* pool name, or data base problem: reject request      */
		return_serial_validate(Acp, REQ_DENIED, to);
		(void) strcat ( Message, (char *)Name ) ;
		(void) strcat ( Message, ",bad outbound poolname");
		log_message(inet, logid, port, service, EVENT_REJECT, Message);
		if (debug) {
			printf("ipx_check_dialback log: %s.\n",Message); 
			fflush(stdout);  
			}
		goto finished;
		}
	else if (strlen(Phone) == 0) {
		/* prompt user for telephone number */
		got_str = promptstring(Acp, Phone, ACP_PHONEPROMPT, 1,
					INPUT_TIMEOUT, to);
		if (debug) {
			printf("ipx_check_dialback: got_str=%d, phone=%s.\n",
					got_str,Phone); 
			fflush(stdout);  
			}
		}
	if (strlen(Phone) == 0)	{
		/* error: don't have the phone number */
		return_serial_validate(Acp, REQ_DENIED, to);
		(void) strcat ( Message, (char *)Name ) ;
		(void) strcat ( Message, ",no phone number");
		log_message(inet, logid, port, service, EVENT_REJECT, Message);
		if (debug) {
			printf("ipx_check_dialback log: %s.\n",Message); 
			fflush(stdout);  
			}
		}
	else	{
		/* everything looks good */
                error = get_user_profile(Name,&up);
		(void)return_serial_validate(Acp, REQ_DIALB_GRANT, to);

		if (debug) {
			printf("ipx_check_dialback: calling acp_request_dialout\n");
			fflush(stdout);
			}

		Message[0] = 0;
		(void) strcat ( Message, (char *)Name ) ;
		(void) strcat ( Message, ",dialback request");
		log_message(inet, logid, port, service, EVENT_LOGIN, Message);
		if (debug) {
			printf("ipx_check_dialback log: %s.\n",Message); 
			fflush(stdout);  
			}

		errno = -1;

		/*
		 * give the requesting annex time to disconnect
		 * and release the port. This is necessary in case
		 * both the dial in and the dial out go through
		 * the same annex port: if we don't sleep the
		 * dialout process will find the port is busy
		 * disconnecting. Wait a few seconds.
		 */
		(void)sleep(6);

		do {
		    int errno2;

		    /* get Annex addr.and port set from pool member */
		    (void) bcopy(pool_info.pe_portmap, Port_mask,LEN_PORT_MASK);
		    target_inet = pool_info.pe_hostaddr;

		    /* send request to appropriate annex */

                    errno = acp_request_dialout(target_inet, Name, Access_code,
				Phone, Job, Port_mask, &dialb_port, service, Netnum);

		    if(debug) {
			printf("acp_request_dialout returned errno=%d\n",errno);
			fflush(stdout);  
			}
		    if (errno == DIAL_SUCC)
			break;

		    /* dialback request failed: try next pool member */
		    errno2 = get_next_pool_entry(&pool_info);
		    if (errno2 == ACPU_ESUCCESS)
			errno = errno2;
		    if (debug) {
			printf("get_next_pool_entry ret %d\n",errno2);
			fflush(stdout);
			}
		} while (errno == ACPU_ESUCCESS);

                dialback_log(inet, port, Name, Phone, target_inet, 
				logid, dialb_port, service, Message);

		if (debug) {
			printf("ipx_check_dialback log: %s.\n",Message); 
			fflush(stdout);  
			}
		}
	}
finished:
  return;
}


void
dialback_log(inet, port, Name, Phone, target_inet, logid, 
		dialb_port, service, Message)
UINT32 inet;
char	*Name, *Phone;
UINT32 target_inet;
UINT32 logid;
unsigned short dialb_port;
unsigned short service;
char    *Message;
{
	/* do message logging */
	Message[0] = 0;
	switch(errno) {
	case DIAL_SUCC:
		(void) sprintf(Message,"%s, tel:%s", Name, Phone);
		log_message(target_inet, logid, dialb_port,
			service, EVENT_DIAL, Message);
		break;
	case DIAL_EBUSY:
		(void) sprintf(Message,"%s, ports busy", Name);
		log_message(inet, logid, port,
			service, EVENT_DIAL, Message);
		break;
	case DIAL_ETIMEDOUT:
		(void) sprintf(Message,"%s, timed out", Name);
		log_message(inet, logid, dialb_port,
			service, EVENT_DIAL, Message);
		break;
	case DIAL_TIME:
		(void) sprintf(Message,"%s, srpc timed out", Name);
		log_message(inet, logid, port,
			service, EVENT_DIAL, Message);
		break;
	case DIAL_EINVAL:
		(void) sprintf(Message,"%s, unknown modem", Name);
		log_message(inet, logid, dialb_port,
			service, EVENT_DIAL, Message);
		break;
	case DIAL_EIO:
		(void) sprintf(Message,"%s, I/O error", Name);
		log_message(inet, logid, dialb_port,
			service, EVENT_DIAL, Message);
		break;
	default:
		(void) sprintf(Message,"%s, failed", Name);
		log_message(inet, logid, port,
			service, EVENT_DIAL, Message);
		break;
	}
  return;
}

 /* ndbm database check for userdb */


int is_slip (user) 
char *user;
{
	DBM *userdb;
	datum inbuff, key;
	FILE *fp;
	
	char *delim = ":";
	char *tok;
	char *slip = "sS";
	char *expired = "eE";

	char *file="/etc/admin/userdb\0";

	int i, l;
	int slipuser=0;

	userdb = dbm_open(file,DBM_INSERT,0);

/*	i = dbm_error(userdb); */

	key.dptr = user;
	key.dsize = strlen(user);

	inbuff = dbm_fetch(userdb, key);
	if (inbuff.dptr != NULL) {

		/* Get token for additional accounts */
		tok = (char *)strtok(inbuff.dptr, delim);
		for (i=1; i<=17; i++) {
			tok = (char *)strtok(NULL, delim);
		}
		l = strlen(tok);
		i = strcspn(tok,slip);

		/* If it is SLIP, check expired flag */
		if (i < l) {
 			for (i=1; i<=6; i++) {
				tok = (char *)strtok(NULL,delim);
			}
			l = strlen(tok);
			i = strcspn(tok,expired);
			if (i == l) slipuser = -1;

		}
	}

	dbm_close(userdb);
	return slipuser;
}

int check_server (user, Message)
	char *user, *Message;
	{
	FILE *fp;
	char buf[90];
	char cmd[10];
	char luser[10], paduser[11];
	char *cr = "\n";
	char *sp = " ";
	register int s;
	int found = 0;
	int i,l;
	struct hostent *hp;
	struct sockaddr_in sin;
	
	char *no_act = "No such activity.";

	char *hostname[NUM_SERVERS] = { "ts1", "ts2" };

	l = strlen(user);
	if ( (l > 9) || (l == 0) ) return(found);

	strcpy(cmd,user);
	strcat (cmd,cr);

	strcpy(paduser, user);
	strcat(paduser,sp);
	l = strlen(paduser);
			
	for (i=0; i<NUM_SERVERS; i++) {
		if ((hp = gethostbyname(hostname[i])) == NULL) {
			sprintf(Message, "Unknown host %s", hostname[i]);
			return (-1);
		}
	
		if ((s = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
			sprintf(Message, "Socket error %s",errno);
			return (-1);
		}
	
		sin.sin_family = AF_INET;
		sin.sin_port = htons(79);
		bcopy(hp->h_addr, &sin.sin_addr, hp->h_length);
	
		signal(SIGALRM, timeout);
		if (setjmp(env) == 0) {
			alarm(3);
			if (connect(s, &sin, sizeof(sin)) < 0) {
				sprintf(Message,"Connect timeout error %i for host %s",errno, 
					hostname[i]);
				return(-1);
			}
			alarm(0);
		} else {
			sprintf(Message,"Timeout contacting %s",hostname[i]);
			return(-1);
		}			
	
		fp = fdopen(s,"r");
	
		send (s, cmd, strlen(cmd), 0);

		/* Check for 'No such activity' */
		if ( fgets(buf, 80, fp) == NULL) return(found);
		if (strncmp(buf,no_act,strlen(no_act)) != 0) {
		
			while (fgets(buf, 80, fp) != NULL) {
				strncpy(luser,buf+10,9);
				if ( strncmp(luser, paduser, l) == 0) {
					found++;
				}
			}
		}
	
		close(s);

	}  /* end for */

	return(found);
	
}
	
timeout(int sig) {
	signal (sig, SIG_IGN);
	longjmp(env, 1);
}
	
