/*
 *****************************************************************************
 *
 *        Copyright 1989, Xylogics, Inc.  ALL RIGHTS RESERVED.
 *
 * ALL RIGHTS RESERVED. Licensed Material - Property of Xylogics, Inc.
 * This software is made available solely pursuant to the terms of a
 * software license agreement which governs its use.
 * Unauthorized duplication, distribution or sale are strictly prohibited.
 *
 * Include file description:
 *	%$(description)$%
 *
 * Original Author: %$(author)$%	Created on: %$(created-on)$%
 *
 * Revision Control Information:
 *
 * $Header: /annex/release9.0/src/erpcd/RCS/acp_policy.h,v 1.42.102.4 1995/02/23 15:00:54 bullock Exp $
 *
 * This file created by RCS from:
 * $Source: /annex/release9.0/src/erpcd/RCS/acp_policy.h,v $
 *
 * Revision History:
 *
 * $Log: acp_policy.h,v $
 * Revision 1.42.102.4  1995/02/23  15:00:54  bullock
 * spr.4261 - Add MASK_IPX for masking the ipx command
 *
 * Revision 1.42.102.3  1994/11/17  16:00:17  defina
 * Added some comments for shadow/passwd selection.
 *
 * Revision 1.42.102.2  1994/09/20  15:47:22  bullock
 * Fix mismatched services between this file and acp.h in oper/acp.  It is
 * now in keeping with R9.1 so we don't have to change it again later.
 *
 * Revision 1.42.102.1  1994/09/20  15:20:50  defina
 * Added the DIALBACK_DELAY parameter and text.
 *
 * Revision 1.42  1994/08/26  17:45:31  sasson
 * SPR 3325: added service names for dynamic dial.
 *
 * Revision 1.41  1994/07/29  13:04:21  defina
 * Put in equate for USE_F_LOCK and documented its usage.
 *
 * Revision 1.40  1994/05/17  17:08:16  reeve
 * Merge from IPX tree.
 *
 * Revision 1.39  1994/04/15  16:36:12  sasson
 * SPR 2613: added defines for the securID code fix.
 *
 * Revision 1.38  1994/03/10  14:38:03  carlson
 * SPR 2113 -- Made FASTRETRY the default and added optional switches
 * for hook-callback code.
 *
 * Revision 1.37  1994/02/23  14:52:00  carlson
 * SPR 2368 -- removed DIALUP_SLIP option.
 *
 * Revision 1.36  1994/01/21  10:02:19  carlson
 * Added FTP service to list of known services.
 *
 * Revision 1.35  1993/12/30  13:05:36  carlson
 * SPR 2084 -- made compatible with 64 bit machines.
 *
 * Revision 1.34  1993/12/15  12:56:50  carlson
 * SPR 2068 -- added a simple means of enabling both SecurID and ACP
 * at the same time.
 *
 * Revision 1.33  1993/12/11  15:10:45  carlson
 * SPR 2226 -- added USE_SECONDS option.
 * SPR 1505 -- added USE_SYSLOG and USE_LOGFILE options.
 * Also added USE_ANAME option to decode Annex peer names via host tables.
 *
 * Revision 1.31.1.2  1993/12/07  12:16:04  carlson
 * Moved FASTRETRY, USESHADOW, NATIVEPASSWD and NATIVESHADOW here.
 * SPR 2226 -- added USE_SECONDS option.
 * SPR 1505 -- added USE_SYSLOG and USE_LOGFILE options.
 * Also added USE_ANAME option to decode Annex peer names via host tables.
 *
 * Revision 1.31.1.1  1993/12/07  09:31:48  carlson
 * SPR 1016 -- added variable install_dir and eservices definition.
 *
 * Revision 1.31  1993/10/19  11:41:21  sasson
 * Renamed LOG_MASK to ACP_LOG_MASK to avoid conflict in <syslog.h>.
 *
 * Revision 1.30  1993/10/15  17:02:41  bullock
 * spr.1940 - Add a "none" mask for acp_userinfo climask usage
 *
 * Revision 1.29  93/08/24  17:26:03  sasson
 * Added define option (DEFAULT_NO_USERINFO) that allows users who do not
 * have entries in acp_userinfo to dial in into the annex.
 * 
 * Revision 1.28  93/07/30  18:56:11  barnes
 * change username/password prompts when building acp with the
 * securid features
 * 
 * Revision 1.27  1993/06/29  17:56:55  reeve
 * Added new message for cli access, but no dialback.
 * [SPR 1787] Reviewed by David Wang.
 *
 * Revision 1.26  93/05/24  13:35:44  wang
 * Undefine PORT_PASSWORD if SECURID define.
 * 
 * Revision 1.25  93/03/31  14:45:57  bockenek
 * Added mask bits for cli commands connect, services, ppp and arap.
 * 
 * Revision 1.24  93/03/30  01:11:27  grant
 * Added appletalk user profile call.
 * 
 * Revision 1.23  93/03/08  10:24:36  sasson
 * Changed the "Dialback request accepted" message to match the
 * funtional spec.
 * 
 * Revision 1.22  93/03/05  18:29:31  sasson
 * Added support for dialback security.
 * 
 * Revision 1.21  93/03/03  07:58:58  barnes
 * add support for securid card
 * 
 * Revision 1.20  93/02/19  13:36:02  reeve
 * Added new services for dynamic dialing.
 * 
 * Revision 1.19  93/01/26  10:51:28  carlson
 * Updated prompt strings to a reasonable length and added defines
 * necessary to form system password file names.
 * 
 * Revision 1.18  93/01/21  11:06:36  carlson
 * SPR 1285 -- added support for shadow file.  Also added various
 * warning messages for password and account aging, and added support
 * for using standard libraries for passwd and shadow file access.
 * 
 * Revision 1.17  92/03/09  18:09:31  carlson
 * Added INPUT_POLL_TIMEOUT for FASTRETRY option.
 * 
 * Revision 1.16  92/02/28  14:33:39  barnes
 * add logging for LAT connect
 * 
 * Revision 1.15  92/01/07  16:47:42  carlson
 * Added defines for SERVICE_SLIP/PPP.
 * 
 * Revision 1.14  91/11/15  13:21:13  reeve
 * Added definitions for a new service type and two new event types.
 * All for dialup address requests.
 * 
 * Revision 1.13  91/04/08  22:07:36  emond
 * Accommodate fix to INSTALL_DIR so it will work on SysVs
 * 
 * Revision 1.12  90/04/27  11:15:19  loverso
 * Add #define for DIALUP_SLIP - but leave it commented out.
 * 
 * Revision 1.11  90/04/10  10:07:11  loverso
 * make sure MASK_SLIP is included
 * 
 * Revision 1.10  90/03/06  23:08:34  loverso
 * Add macros for PORT_PASSWORD
 * 
 * Revision 1.9  89/04/05  12:21:10  loverso
 * Changed copyright notice
 * 
 * Revision 1.8  88/08/05  14:40:34  harris
 * Added define for USER_VALIDATION - user name/password prompting.
 * 
 * Revision 1.7  88/06/15  16:46:11  harris
 * Added a define, MAX_KEYS - number of encryption key entries allowed.
 * 
 * Revision 1.6  88/06/10  15:48:48  harris
 * Change acp_secure to acp_keys.
 * 
 * Revision 1.5  88/05/05  11:33:36  harris
 * Changes for secure annex file, ACP_SECURE, and password cacheing.
 * 
 * Revision 1.4  88/04/15  12:09:15  mattes
 * 4.0 integration
 * 
 * Revision 1.3  87/11/06  13:45:32  parker
 * Made install directory setable
 * 
 * Revision 1.2  87/08/13  17:48:34  parker
 * fixed cli login to use MAX_RETRIES
 * 
 * Revision 1.1  87/07/28  11:53:00  parker
 * merged in security
 * 
 *
 * This file is currently under revision by:
 *
 * $Locker:  $
 *
 *  DATE:	$Date: 1995/02/23 15:00:54 $
 *  REVISION:	$Revision: 1.42.102.4 $
 *
 ****************************************************************************
 */


/* select user name/password validation	*/
#define	USER_VALIDATION	1

/*
 * Uncomment this line to compile a "fast-retry" version of the security
 * code.  This option requires that all served Annexes are running R6.2
 * or later of the operational image.  Otherwise, duplicate prompts may
 * result.
 */
#define FASTRETRY 1

/*
 * Uncomment this line to compile in the shadow password file option.
 * (This enables password aging and protection of encrypted passwords.)
 *
 * This feature, when enabled, will use either the Annex acp_shadow
 * file that resides in the annex install directory or the standard system
 * shadow file usually found in the /etc directory.
 * See shadow/passwd file selection below.
 */
/* #define USESHADOW 1 */

/*
 * Uncomment this line to compile for SunOS 4.1.x with C2 security
 * or adjunct passwords. Will use the native SunOS pwdauth call.
 */
#define SUNADJUNCT 1

/*
 * Uncomment these lines to compile with the normal /etc/passwd
 * searching routines, rather than the default acp_passwd routines.
 * (This will link acp into NIS (erstwhile YP) on those systems that
 * support it.)
 *
 * PASSWD/SHADOW MATRIX:
 *
 * ann = The Annex install directory
 *
 * USESHADOW NATIVEPASSWD NATIVESHADOW      DIR/FILES
 * 
 * 0         0            0  No Shadow       ann/acp_passwd
 * 0         0            1  Bad Config
 * 0         1            0  No shadow       etc/passwd      
 * 0         1            1  Bad Config
 * 1         0            0  Shadow Enabled  ann/acp_passwd ann/acp_shadow
 * 1         0            1  Shadow enabled  ann/acp_passwd etc/shadow
 * 1         1            0  Shadow enabled  etc/passwd ann/acp_shadow
 * 1         1            1  Shadow enabled  etc/passwd etc/shadow
 * 
 */
/* #define NATIVEPASSWD 1 */
/* #define NATIVESHADOW 1 */

/*
 * Uncomment this line to substitute seconds-since-1970 (raw time
 * format) in place of year/month/day/hour/minute/second in the log
 * file.  This form is more amenable to automatic parsing.
 */
/* #define USE_SECONDS 1 */

/*
 * Uncomment this line to select the use of the standard syslog(3)
 * facility in addition to or in place of the logfile -- the value of
 * "USE_SYSLOG" is used to identify the daemon.  (Comment the second
 * line out to disable the normal acp log file.)
 */
/* #define USE_SYSLOG "annex" */
#define USE_LOGFILE 1

/*
 * Uncomment this line to select the F_LOCK method to lock the acp_logfile
 * for updating.
 *
 * A file must be locked for update in order to block other processes from
 * writing to it simultaneously.
 *
 * F_LOCK - Passing the F_LOCK as the cmd value when making system lockf call
 * is the most efficient and preferred maner to lock a file for exclusive write * access. In this scenario a process is put to sleep until the resource is
 * available. Once available the process is preempted owning the resource.
 * available. Once available the process is preempted owning the resource.
 *
 * T_LOCK - When the T_LOCK cmd argument is passed, the process must
 * repeatedly send the lockf call the until the resource is available.
 * Once available the system call returns a success and the resource is
 * acquired.
 *
 * The F_LOCK cmd has been determined to be faulty on many hosts. Failures
 * can not be narrowed down to any particular hardware manufacturer or UNIX
 * system. There are to many OS revs and varables to sense the correct lockf
 * method to use at installation time. The default, T_LOCK was chosen simply
 * because it has been proven reliable. SEE 'log_message()'
 */
/* #define USE_F_LOCK 1 */

/*
 * Uncomment this line to use decoded Annex peer names, rather than
 * numeric IP addresses, in the log file and in syslogging.
 */
#define USE_ANAME 1

#ifdef SECURID_CARD

/*
 * This macro can be changed to select the ports on which the SecurID
 * system is used.  For example, using this definition will enable
 * SecurID on ports 2 and 3, and normal ACP security on the other ports:
 *	#define USE_SECURID_CHECK (port == 2 || port == 3)
 * As defined below (just a '1'), all ports get SecurID security.
 */
#define USE_SECURID_CHECK 1

#else /* !SECURID_CARD */

/* enable (currently unsupported) "Port Password:" feature */
#define	PORT_PASSWORD 1

#endif

/*
 * Enable basic usage of the CLI "hooks" code.  This is necessary for
 * the demonstration below.
 */
#define ALLOW_HOOKS 1

/*
 * Enable demonstration CLI "hooks" code.  This creates two fake "users"
 * available from the "Annex username" prompt -- "library" will cause an
 * immediate telnet to a machine called "library" and "help" will print
 * out some help text and return to the "Annex username" prompt.
 *
 * You should remove and replace this code in acp_policy.c with your own
 * policy code.
 */
/* #define ALLOW_HOOKS_DEMO  */

#define	PATHSZ	256		/* Max size of path name */

/*	Define pathnames password and shadow files	*/

#ifdef NATIVEPASSWD
#define ACP_PASSWD(str)	strcpy(str,"/etc/passwd")
#define ACP_PTMP(str)	strcpy(str,"/etc/passwd.tmp")
#else
#define ACP_PASSWD(str)	sprintf(str,"%s/acp_passwd",install_dir)
#define ACP_PTMP(str)	sprintf(str,"%s/acp_ptmp",install_dir)
#endif

#ifdef NATIVESHADOW
#define ACP_SHADOW(str)	strcpy(str,"/etc/shadow")
#define ACP_STMP(str)	strcpy(str,"/etc/shadow.tmp")
#define ACP_LOCKFILE(str) strcpy(str,"/etc/.pwd.lock")
#else
#define ACP_SHADOW(str)	sprintf(str,"%s/acp_shadow",install_dir)
#define ACP_STMP(str)	sprintf(str,"%s/acp_stmp",install_dir)
#define ACP_LOCKFILE(str) sprintf(str,"%s/.pwd.lock",install_dir)
#endif

/*	define pathname of accounting file		*/
#define ACP_LOGFILE(str) \
	sprintf(str,"%s/acp_logfile",install_dir)

/*	define pathname for restrictions file		*/
#define ACP_RESTRICT(str) \
	sprintf(str,"%s/acp_restrict",install_dir)

/*	define pathanme for annex acp_keys file 	*/
#define	ACP_KEYS(str) \
	sprintf(str,"%s/acp_keys",install_dir)

/*	define pathanme for annex dialup addresses file */
#define ACP_DIALUP(str) \
	sprintf(str,"%s/acp_dialup",install_dir)

/*	define pathname for user profile file 		*/
#define ACP_USERINFO(str) \
	sprintf(str,"%s/acp_userinfo",install_dir)

#define ACP_ESERVICES(str) \
	sprintf(str,"%s/eservices",install_dir)

/*	define sizes of internal encryption tables	*/

#define MAX_WILD	32
#define	MAX_TAME	96
#define MAX_KEYS	128

/*	define messages used by default application	*/

#define ACP_USERPROMPT	"login: "
#define ACP_PASSPROMPT	"Password: "
#define ACP_PERMGRANTD	"\nPermission granted\n"
#define ACP_PERMDENIED	"\007\nPermission denied\n"
#define ACP_INCORRECT	"\nUsername/Password Incorrect\n"
#ifdef SECURID_CARD
#define SID_USERPROMPT	"Username: "
#define SID_PASSPROMPT	"Enter PASSCODE: "
#define SID_PERMGRANTD	"\nPASSCODE accepted\n"
#define SID_PERMDENIED	"\007\nAccess Denied\n"
#define SID_INCORRECT	"\nUsername/PASSCODE Incorrect\n"
#define SID_LOGNEWPIN_1 "\nWait for the code on your card to change,\n"
#define SID_LOGNEWPIN_2 "then enter PASSCODE including the new PIN.\n"
#endif /* SECURID_CARD */

#define ACP_TIMEDOUT	"\007\nLogin Timed Out\n"
#define ACP_WARNING	"\007\nYour password will expire in %ld days unless changed.\n"
#define ACP_WARNINGM	"\007\nYour password expires after tomorrow unless changed.\n"
#define ACP_WARNINGT	"\007\nYour password expires after today unless changed.\n"
#define ACP_AWARNING	"\007\nYour account will expire in %ld days.\n"
#define ACP_AWARNINGM	"\007\nYour account expires after tomorrow.\n"
#define ACP_AWARNINGT	"\007\nYour account expires after today.\n"
#define ACP_EXPIRED	"Your password has expired.\n"
#define ACP_NEWPASS	"Enter a new password:  "
#define ACP_NEWPASS2	"Re-enter new password:  "
#define ACP_PASSMATCH	"Entered passwords do not match.  Try again.\n"
#define ACP_ACCESSCODEPROMPT	"Access Code: "
#define ACP_PHONEPROMPT		"Telephone Number: "
#define ACP_DIALBACKGRANTD	"\nRequest accepted, dialback in progress...\n"
#define ACP_CLINODIALBACK	"\nPermission granted, no dialback\n"

#ifdef SECURID_CARD
/* 	define messages used by Securid Card application	*/

#define SID_NEXTCODEPROMPT      "Enter next card code: "
#define SID_PINCHAR     "characters"
#define SID_PINDIGIT    "digits"
#define SID_PINSIZE     "%d"
#define SID_PINSZRANGE  "%d to %d"
#define SID_NEWPINPROMPT        "Enter your new PIN containing %s %s,\n"
#define SID_OR          "\t\tor\n"
#define SID_NEWPIN_2    "Press <Return> to generate new PIN and display it\n"
#define SID_NEWPIN_3    "      <Ctrl d> <Return> to leave your card in New-PIN mode.\n"
#define SID_SYSGENPIN   "\t\t%s\n"
#define SID_PINREENTRY  "Please re-enter PIN: "
#endif /* SECURID_CARD */

#ifdef PORT_PASSWORD
/* only used if PORT_PASSWORD set and a port password exists in acp_passwd */
#define ACP_PORTPROMPT	"Port password: "
#endif

/*
 *    DIALBACK DELAY - When a user requests a valid dialback connection,
 *    ACP checks authentication, then instructs the Annex to
 *    issue the dialout. This delay is the number of seconds ACP waits prior
 *    to sending the acp_request_dialout back to the Annex. The delay is
 *    required for the case where the Annex dial in and dial out ports
 *    are configured to be the same. It gives the target Annex time to
 *    disconnect from the port before starting the dialout. If enough time
 *    is not allotted, the dialout process will find that the port is busy.
 *    Set this parameter to a higher value if a dialback attempt causes a
 *    "port is busy" message in the acp_logfile.
 */
#define DIALBACK_DELAY 6

/*	miscellaneous defines for default application	*/

#define INPUT_TIMEOUT		30
#define INPUT_POLL_TIMEOUT	3
#define RETRIES_MAX		3

/*	define bit to disable each maskable CLI command	*/

#define MASK_BG		0x00000001
#define MASK_CALL	0x00000002
#define MASK_FG		0x00000004
#define MASK_HANGUP	0x00000008
#define MASK_HELP	0x00000010
#define MASK_HOSTS	0x00000020
#define MASK_JOBS	0x00000040
#define MASK_KILL	0x00000080
#define MASK_NETSTAT	0x00000100
#define MASK_RLOGIN	0x00000200
#define MASK_STATS	0x00000400
#define MASK_STTY	0x00000800
#define MASK_TELNET	0x00001000
#define MASK_WHO	0x00002000
#define MASK_LOCK	0x00004000
#define MASK_SU		0x00008000
#define MASK_SLIP	0x00010000
#define MASK_CONNECT	0x00020000
#define MASK_SERVICES	0x00040000
#define MASK_PPP	0x00080000
#define MASK_ARAP	0x00100000
#define MASK_IPX	0x00200000
#define MASK_NONE	0x80000000

/*	define cli command mask		*/

#define CLI_MASK	(UINT32)0

/*	define service types		*/

/*      NOTE: NOTE: NOTE: NOTE: NOTE:
 *	THESE SHOULD MATCH EXACTLY WITH
 *	THE SERVICES IN ../oper/acp/acp.h
 */
#define NSERVICES		24
#define SERVICE_SECURITY	0
#define SERVICE_CLI		1
#define SERVICE_CALL		2
#define SERVICE_RLOGIN		3
#define SERVICE_TELNET		4
#define SERVICE_PORTS		5
#define SERVICE_DIALUP		6
#define SERVICE_SLIP		7
#define SERVICE_PPP		8
#define	SERVICE_CONNECT		9
#define	SERVICE_SLIP_DYNDIAL	10
#define	SERVICE_PPP_DYNDIAL	11
#define	SERVICE_DIALBACK	12
#define	SERVICE_ARAP		13
#define SERVICE_FTP		14
#define SERVICE_CLI_HOOK	15
#define       SERVICE_IPX             16
#define       SERVICE_IPX_DIALBACK    17
#define	SERVICE_RCF		18
#define SERVICE_PPP_TMOUT		19
#define SERVICE_PPP_DYNDIAL_TMOUT	20
#define SERVICE_SLIP_TMOUT		21
#define SERVICE_SLIP_DYNDIAL_TMOUT	22
#define SERVICE_VMS		23

/*	define event logging (accounting) parameters	*/

#define NEVENTS		8
#define EVENT_BOOT	0
#define EVENT_LOGIN	1
#define EVENT_REJECT	2
#define EVENT_LOGOUT	3
#define EVENT_TIMEOUT	4
#define EVENT_PROVIDE	5
#define EVENT_NOPROVIDE	6
#define EVENT_DIAL	7

#define ACP_LOG_MASK	0640	/*  umask when creating ACP_LOGFILE  */

#define IPX_CHARGE_BACK_TOK     "charge_back"
#define IPX_ACCESS_CODE_TOK     "ipx"

/* Just in case ... */
#ifdef NOT_ANSI_LIB
# define LOG_FORMAT_S	"%s:%8.8x:#%2.2d:%10.10d:%s:%s:%s\n"
# define LOG_FORMAT	"%s:%8.8x:#%2.2d:%2.2d%2.2d%2.2d:%2.2d%2.2d%2.2d:%s:%s:%s\n"
#else /* !NOT_ANSI_LIB */
# define LOG_FORMAT_S	"%s:%8.8lx:#%2.2d:%10.10ld:%s:%s:%s\n"
# define LOG_FORMAT	"%s:%8.8lx:#%2.2d:%2.2d%2.2d%2.2d:%2.2d%2.2d%2.2d:%s:%s:%s\n"
#endif /* NOT_ANSI_LIB */

#ifndef TRUE
#define TRUE 1
#define FALSE 0
#endif

/*
 * If the native passwd(5) functions are being used, then convert the
 * references to the acp routines over to normal library calls.
 */
#ifdef NATIVEPASSWD
#define setacppw() setpwent()
#define endacppw() endpwent()
#define getacppw() getpwent()
#else
int setacppw();
void endacppw();
struct passwd *getacppw();
#endif
#ifdef NATIVESHADOW
#define setacpsp() setspent()
#define endacpsp() endspent()
#define getacpsp() getspent()
#define lckacpf() lckpwdf()
#define ulckacpf() ulckpwdf()
#else
int setacpsp();
void endacpsp();
struct spwd *getacpsp();
int lckacpf(),ulckacpf();
#endif

/* These are in getacppw.c */
extern char passwd_name[],ptmp_name[];
extern char shadow_name[],stmp_name[];
extern char lock_name[];

/* These are in erpcd.c */
extern char *root_dir,*install_dir;

/*
 * When DEFAULT_NO_USERINFO is set to 1, a user is allowed to dial in into
 *					 the Annex even if the user does not
 *					 have an entry in the acp_userinfo file.
 *					 An access_code does not need to be
 *					 assigned to all users. Notice that
 *					 users requesting dialback must have
 *					 a valid entry in acp_userinfo.
 *
 * When DEFAULT_NO_USERINFO is set to 0, all users are required to have an
 *					 access_code defined in acp_userinfo
 *					 for increased security. All users
 *					 will be prompted for the access code
 *					 during authentication.
 */
#define	DEFAULT_NO_USERINFO	1

/*
 * Set the number of server to check, if we want to do maximum log-in checking
  */
  
#define NUM_SERVERS 2

